@@ -33,6 +33,7 @@
 #include "test/support/src/helpers.h"
 #include "test/support/src/vfs_helpers.h"
 #include "tiledb/common/common.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/array_schema/tile_domain.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/query/legacy/read_cell_slab_iter.h"
@@ -63,6 +64,8 @@ struct ReadCellSlabIterFx {
   const char* ARRAY_NAME = "read_cell_slab_iter";
   tiledb_array_t* array_ = nullptr;
 
+  shared_ptr<MemoryTracker> tracker_;
+
   ReadCellSlabIterFx();
   ~ReadCellSlabIterFx();
 
@@ -82,7 +85,8 @@ struct ReadCellSlabIterFx {
 };
 
 ReadCellSlabIterFx::ReadCellSlabIterFx()
-    : fs_vec_(vfs_test_get_fs_vec()) {
+    : fs_vec_(vfs_test_get_fs_vec())
+    , tracker_(tiledb::test::create_test_memory_tracker()) {
   // Initialize vfs test
   REQUIRE(vfs_test_init(fs_vec_, &ctx_, &vfs_).ok());
 
@@ -162,7 +166,8 @@ void ReadCellSlabIterFx::create_result_space_tiles(
       tile_coords,
       array_tile_domain,
       frag_tile_domains,
-      result_space_tiles);
+      result_space_tiles,
+      tiledb::test::get_test_memory_tracker());
 }
 
 void set_result_tile_dim(
@@ -256,10 +261,10 @@ TEST_CASE_METHOD(
   shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
       HERE(),
       nullptr,
-      nullptr,
       array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
+      tiledb::test::create_test_memory_tracker(),
       true);
   fragments.emplace_back(std::move(fragment));
 
@@ -330,10 +335,10 @@ TEST_CASE_METHOD(
   shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
       HERE(),
       nullptr,
-      nullptr,
       array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
+      tiledb::test::create_test_memory_tracker(),
       true);
   fragments.emplace_back(std::move(fragment));
 
@@ -408,10 +413,10 @@ TEST_CASE_METHOD(
     shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
         HERE(),
         nullptr,
-        nullptr,
         array_->array_->array_schema_latest_ptr(),
         URI(),
         std::make_pair<uint64_t, uint64_t>(0, 0),
+        tiledb::test::create_test_memory_tracker(),
         true);
     fragments.emplace_back(std::move(fragment));
   }
@@ -491,10 +496,10 @@ TEST_CASE_METHOD(
     shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
         HERE(),
         nullptr,
-        nullptr,
         array_->array_->array_schema_latest_ptr(),
         URI(),
         std::make_pair<uint64_t, uint64_t>(0, 0),
+        tiledb::test::create_test_memory_tracker(),
         true);
     fragments.emplace_back(std::move(fragment));
   }
@@ -510,9 +515,12 @@ TEST_CASE_METHOD(
 
   // Create result coordinates
   std::vector<ResultCoords> result_coords;
-  ResultTile result_tile_2_0(1, 0, array_schema);
-  ResultTile result_tile_3_0(2, 0, array_schema);
-  ResultTile result_tile_3_1(2, 1, array_schema);
+  ResultTile result_tile_2_0(
+      1, 0, *fragments[0], tiledb::test::get_test_memory_tracker());
+  ResultTile result_tile_3_0(
+      2, 0, *fragments[0], tiledb::test::get_test_memory_tracker());
+  ResultTile result_tile_3_1(
+      2, 1, *fragments[1], tiledb::test::get_test_memory_tracker());
 
   set_result_tile_dim(
       array_schema, result_tile_2_0, "d", 0, {{1000, 3, 1000, 5}});
@@ -711,10 +719,10 @@ TEST_CASE_METHOD(
   shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
       HERE(),
       nullptr,
-      nullptr,
       array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
+      tiledb::test::create_test_memory_tracker(),
       true);
   fragments.emplace_back(std::move(fragment));
 
@@ -897,10 +905,10 @@ TEST_CASE_METHOD(
   shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
       HERE(),
       nullptr,
-      nullptr,
       array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
+      tiledb::test::create_test_memory_tracker(),
       true);
   fragments.emplace_back(std::move(fragment));
 
@@ -1096,10 +1104,10 @@ TEST_CASE_METHOD(
   shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
       HERE(),
       nullptr,
-      nullptr,
       array_->array_->array_schema_latest_ptr(),
       URI(),
       std::make_pair<uint64_t, uint64_t>(0, 0),
+      tiledb::test::create_test_memory_tracker(),
       true);
   fragments.emplace_back(std::move(fragment));
 
@@ -1342,10 +1350,10 @@ TEST_CASE_METHOD(
     shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
         HERE(),
         nullptr,
-        nullptr,
         array_->array_->array_schema_latest_ptr(),
         URI(),
         std::make_pair<uint64_t, uint64_t>(0, 0),
+        tiledb::test::create_test_memory_tracker(),
         true);
     fragments.emplace_back(std::move(fragment));
   }
@@ -1361,8 +1369,10 @@ TEST_CASE_METHOD(
 
   // Create result coordinates
   std::vector<ResultCoords> result_coords;
-  ResultTile result_tile_3_0(2, 0, array_schema);
-  ResultTile result_tile_3_1(2, 1, array_schema);
+  ResultTile result_tile_3_0(
+      2, 0, *fragments[0], tiledb::test::get_test_memory_tracker());
+  ResultTile result_tile_3_1(
+      2, 1, *fragments[1], tiledb::test::get_test_memory_tracker());
 
   set_result_tile_dim(
       array_schema, result_tile_3_0, "d1", 0, {{1000, 3, 1000, 1000}});
