@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB Inc.
  * @copyright Copyright (c) 2016 MIT and Intel Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -34,11 +34,15 @@
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
 #include "test/support/src/serialization_wrappers.h"
+#include "test/support/src/temporary_local_directory.h"
 #include "tiledb/common/common.h"
+#include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/cpp_api/tiledb"
+#include "tiledb/sm/cpp_api/tiledb_experimental"
 #include "tiledb/sm/misc/constants.h"
 
 #include <chrono>
+#include <filesystem>
 #include <iostream>
 #include <sstream>
 #include <thread>
@@ -51,6 +55,9 @@ namespace {
 static const std::string arrays_dir =
     std::string(TILEDB_TEST_INPUTS_DIR) + "/arrays";
 
+static const std::string groups_dir =
+    std::string(TILEDB_TEST_INPUTS_DIR) + "/groups";
+
 template <typename T>
 void set_query_coords(
     const Domain& domain,
@@ -184,10 +191,8 @@ TEST_CASE(
   std::string array_uri(arrays_dir + "/non_split_coords_v1_4_0");
   Array array(ctx, array_uri, TILEDB_READ);
   std::vector<int> subarray = {1, 4, 10, 10};
-  std::vector<int> a_read;
-  a_read.resize(4);
-  std::vector<int> coords_read;
-  coords_read.resize(8);
+  std::vector<int> a_read(4);
+  std::vector<int> coords_read(8);
 
   Query query_r(ctx, array);
   query_r.set_subarray(subarray)
@@ -211,31 +216,33 @@ TEST_CASE(
     "Backwards compatibility: Test reading arrays written with previous "
     "version of tiledb",
     "[backwards-compat][coords]") {
-  Context ctx;
-  std::string compat_folder(arrays_dir + "/read_compatibility_test");
-  if (Object::object(ctx, compat_folder).type() != Object::Type::Group)
-    return;
-
   std::string encryption_key = "unittestunittestunittestunittest";
 
+  Context ctx;
+  tiledb::Config cfg;
+  cfg["sm.encryption_type"] = "AES_256_GCM";
+  cfg["sm.encryption_key"] = encryption_key.c_str();
+  Context ctx_encrypt(cfg);
+  std::string compat_folder(arrays_dir + "/read_compatibility_test");
+  if (Object::object(ctx, compat_folder).type() != Object::Type::Group) {
+    return;
+  }
+
   tiledb::ObjectIter versions_iter(ctx, compat_folder);
   for (const auto& group_versions : versions_iter) {
     tiledb::ObjectIter obj_iter(ctx, group_versions.uri());
     for (const auto& object : obj_iter) {
-      tiledb::Config cfg;
-      cfg["sm.encryption_type"] = "AES_256_GCM";
-      cfg["sm.encryption_key"] = encryption_key.c_str();
-      Context ctx_cfg(cfg);
       bool encrypted = false;
       Array* array;
 
-      if (object.type() != Object::Type::Array)
+      if (object.type() != Object::Type::Array) {
         continue;
+      }
 
       // Check for if array is encrypted based on name for now
       if (object.uri().find("_encryption_AES_256_GCM") != std::string::npos) {
         encrypted = true;
-        array = new Array(ctx_cfg, object.uri(), TILEDB_READ);
+        array = new Array(ctx_encrypt, object.uri(), TILEDB_READ);
       } else {
         array = new Array(ctx, object.uri(), TILEDB_READ);
       }
@@ -252,15 +259,16 @@ TEST_CASE(
         }
       }
 
-      if (heterogeneous)
+      if (heterogeneous) {
         continue;
+      }
 
       // Skip domain types that are unsupported with zipped coordinates.
       if (domain.type() == TILEDB_STRING_ASCII) {
         continue;
       }
 
-      auto query = new Query(encrypted ? ctx_cfg : ctx, *array);
+      auto query = new Query(encrypted ? ctx_encrypt : ctx, *array);
 
       std::unordered_map<std::string, tuple<uint64_t*, void*, uint8_t*>>
           buffers;
@@ -273,7 +281,9 @@ TEST_CASE(
         uint8_t* validity = static_cast<uint8_t*>(malloc(sizeof(uint8_t)));
 
         switch (attr.second.type()) {
-          case TILEDB_BLOB: {
+          case TILEDB_BLOB:
+          case TILEDB_GEOM_WKB:
+          case TILEDB_GEOM_WKT: {
             set_buffer_wrapper<std::byte>(
                 query,
                 attribute_name,
@@ -575,7 +585,9 @@ TEST_CASE(
 
         Attribute attribute = array->schema().attribute(buff.first);
         switch (attribute.type()) {
-          case TILEDB_BLOB: {
+          case TILEDB_BLOB:
+          case TILEDB_GEOM_WKB:
+          case TILEDB_GEOM_WKT: {
             REQUIRE(
                 static_cast<std::byte*>(std::get<1>(buffer))[0] ==
                 static_cast<std::byte>(1));
@@ -736,35 +748,38 @@ TEST_CASE(
     "Backwards compatibility: Test reading arrays written with previous "
     "version of tiledb using split buffers",
     "[backwards-compat][split-buffers]") {
-  Context ctx;
-  std::string compat_folder(arrays_dir + "/read_compatibility_test");
-  if (Object::object(ctx, compat_folder).type() != Object::Type::Group)
-    return;
   std::string encryption_key = "unittestunittestunittestunittest";
 
+  Context ctx;
+  tiledb::Config cfg;
+  cfg["sm.encryption_type"] = "AES_256_GCM";
+  cfg["sm.encryption_key"] = encryption_key.c_str();
+  Context ctx_encrypt(cfg);
+  std::string compat_folder(arrays_dir + "/read_compatibility_test");
+  if (Object::object(ctx, compat_folder).type() != Object::Type::Group) {
+    return;
+  }
+
   tiledb::ObjectIter versions_iter(ctx, compat_folder);
   for (const auto& group_versions : versions_iter) {
     tiledb::ObjectIter obj_iter(ctx, group_versions.uri());
     for (const auto& object : obj_iter) {
-      tiledb::Config cfg;
-      cfg["sm.encryption_type"] = "AES_256_GCM";
-      cfg["sm.encryption_key"] = encryption_key.c_str();
-      Context ctx_cfg(cfg);
       bool encrypted = false;
       Array* array;
 
-      if (object.type() != Object::Type::Array)
+      if (object.type() != Object::Type::Array) {
         continue;
+      }
 
       // Check for if array is encrypted based on name for now
       if (object.uri().find("_encryption_AES_256_GCM") != std::string::npos) {
         encrypted = true;
-        array = new Array(ctx_cfg, object.uri(), TILEDB_READ);
+        array = new Array(ctx_encrypt, object.uri(), TILEDB_READ);
       } else {
         array = new Array(ctx, object.uri(), TILEDB_READ);
       }
 
-      auto query = new Query(encrypted ? ctx_cfg : ctx, *array);
+      auto query = new Query(encrypted ? ctx_encrypt : ctx, *array);
 
       std::unordered_map<std::string, tuple<uint64_t*, void*, uint8_t*>>
           buffers;
@@ -775,7 +790,9 @@ TEST_CASE(
         uint8_t* validity = static_cast<uint8_t*>(malloc(sizeof(uint8_t)));
 
         switch (attr.second.type()) {
-          case TILEDB_BLOB: {
+          case TILEDB_BLOB:
+          case TILEDB_GEOM_WKB:
+          case TILEDB_GEOM_WKT: {
             set_buffer_wrapper<std::byte>(
                 query,
                 attribute_name,
@@ -1113,7 +1130,9 @@ TEST_CASE(
 
         Attribute attribute = array->schema().attribute(buff.first);
         switch (attribute.type()) {
-          case TILEDB_BLOB: {
+          case TILEDB_BLOB:
+          case TILEDB_GEOM_WKB:
+          case TILEDB_GEOM_WKT: {
             REQUIRE(
                 static_cast<std::byte*>(std::get<1>(buffer))[0] ==
                 static_cast<std::byte>(1));
@@ -1250,12 +1269,9 @@ TEST_CASE(
   // Read using upgraded version
   Array array_read1(ctx, array_name, TILEDB_READ);
   std::vector<int> subarray_read1 = {1, 4, 10, 10};
-  std::vector<int> a_read1;
-  a_read1.resize(4);
-  std::vector<int> d1_read1;
-  d1_read1.resize(4);
-  std::vector<int> d2_read1;
-  d2_read1.resize(4);
+  std::vector<int> a_read1(4);
+  std::vector<int> d1_read1(4);
+  std::vector<int> d2_read1(4);
 
   Query query_read1(ctx, array_read1);
   query_read1.set_subarray(subarray_read1)
@@ -1326,12 +1342,9 @@ TEST_CASE(
   // Read again
   Array array_read2(ctx, array_name, TILEDB_READ);
   std::vector<int> subarray_read2 = {1, 4, 10, 10};
-  std::vector<int> a_read2;
-  a_read2.resize(4);
-  std::vector<int> d1_read2;
-  d1_read2.resize(4);
-  std::vector<int> d2_read2;
-  d2_read2.resize(4);
+  std::vector<int> a_read2(4);
+  std::vector<int> d1_read2(4);
+  std::vector<int> d2_read2(4);
 
   Query query_read2(ctx, array_read2);
   query_read2.set_subarray(subarray_read2)
@@ -1363,4 +1376,111 @@ TEST_CASE(
   vfs.remove_dir(get_fragment_dir(array_read2.uri()));
   vfs.remove_dir(get_commit_dir(array_read2.uri()));
   vfs.remove_dir(schema_folder);
-}
\ No newline at end of file
+}
+
+template <class T>
+void assert_group_metadata(
+    Group& g,
+    const std::string& key,
+    tiledb_datatype_t expected_datatype,
+    T expected_value) {
+  tiledb_datatype_t datatype;
+  uint32_t value_num;
+  const void* value;
+
+  g.get_metadata(key, &datatype, &value_num, &value);
+  REQUIRE(datatype == expected_datatype);
+  REQUIRE(value_num == 1);
+  REQUIRE(value != nullptr);
+  REQUIRE(*static_cast<const T*>(value) == expected_value);
+
+  std::vector<T> expected_values(7);
+  std::fill(expected_values.begin(), expected_values.end(), expected_value);
+  g.get_metadata(key + "_multi", &datatype, &value_num, &value);
+  REQUIRE(datatype == expected_datatype);
+  REQUIRE(value != nullptr);
+  const T* valueT = static_cast<const T*>(value);
+  REQUIRE(std::vector<T>(valueT, valueT + value_num) == expected_values);
+}
+
+TEST_CASE(
+    "Backwards compatibility: Test reading group metadata written with "
+    "previous version of tiledb",
+    "[backwards-compat][group][metadata]") {
+  Context ctx;
+  std::string compat_folder(arrays_dir + "/read_compatibility_test");
+  if (Object::object(ctx, compat_folder).type() != Object::Type::Group) {
+    return;
+  }
+
+  std::string encryption_key = "unittestunittestunittestunittest";
+
+  tiledb::ObjectIter versions_iter(ctx, compat_folder);
+  for (const auto& group_versions : versions_iter) {
+    if (group_versions.type() != Object::Type::Group) {
+      continue;
+    }
+
+    Group g{ctx, group_versions.uri(), TILEDB_READ};
+
+    if (g.metadata_num() == 0) {
+      continue;
+    }
+
+    assert_group_metadata<uint8_t>(g, "u8", TILEDB_UINT8, 0x77);
+    assert_group_metadata<uint16_t>(g, "u16", TILEDB_UINT16, 0x7777);
+    assert_group_metadata<uint32_t>(g, "u32", TILEDB_UINT32, 0x77777777);
+    assert_group_metadata<uint64_t>(
+        g, "u64", TILEDB_UINT64, 0x7777777777777777);
+  }
+}
+
+TEST_CASE(
+    "Backwards compatibility: Test v1 groups",
+    "[backwards-compat][group][v1]") {
+  Context ctx;
+  VFS vfs(ctx);
+
+  // Copy the group to a temporary directory because we will be modifying it.
+  tiledb::sm::TemporaryLocalDirectory temp_dir;
+  std::filesystem::copy(
+      groups_dir + "/group_v1",
+      temp_dir.path(),
+      std::filesystem::copy_options::recursive);
+
+  // Read the group
+  {
+    Group g{ctx, temp_dir.path(), TILEDB_READ};
+
+    CHECK(g.dump(false) != "");
+    CHECK(g.member_count() == 1);
+  }
+
+  // Add a member to the group
+  {
+    Group g{ctx, temp_dir.path(), TILEDB_WRITE};
+
+    Group::create(ctx, temp_dir.path() + "/subgroup2");
+
+    g.add_member("subgroup2", true, "subgroup2");
+
+    g.close();
+  }
+
+  // Read the group again
+  {
+    Group g{ctx, temp_dir.path(), TILEDB_READ};
+
+    CHECK(g.dump(false) != "");
+    CHECK(g.member_count() == 2);
+    CHECK(g.member(1).name() == "subgroup2");
+  }
+
+  // Read the raw group details files
+  auto children = vfs.ls(temp_dir.path() + "/__group");
+  CHECK(children.size() == 2);
+  std::sort(children.begin(), children.end());
+  CHECK(!tiledb::sm::utils::parse::ends_with(children[0], "_1"));
+  // This is the file written by this test.
+  CHECK(tiledb::sm::utils::parse::ends_with(children[1], "_1"));
+}
