@@ -169,7 +169,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"barney", "wilma"};
-  auto qc = QueryConditionExperimental::create(ctx_, "attr2", values);
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr2", values, TILEDB_IN);
 
   check_read(qc, [](const QCSetsCell& c) {
     return (c.a2 == "barney" || c.a2 == "wilma");
@@ -186,7 +187,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"hack", "pack"};
-  auto qc = QueryConditionExperimental::create(ctx_, "attr5", values);
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr5", values, TILEDB_IN);
 
   check_read(qc, [](const QCSetsCell& c) {
     return (c.a5 == "hack" || c.a5 == "pack");
@@ -218,7 +220,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"wilma", "betty"};
-  auto qc = QueryConditionExperimental::create(ctx_, "attr6", values);
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr6", values, TILEDB_IN);
 
   check_read(qc, [](const QCSetsCell& c) { return (c.a6 == 1 || c.a6 == 3); });
 }
@@ -233,7 +236,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"blue", "umber"};
-  auto qc = QueryConditionExperimental::create(ctx_, "attr3", values);
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr3", values, TILEDB_IN);
 
   check_read(qc, [](const QCSetsCell& c) {
     return (c.a3 == "blue" || c.a3 == "umber");
@@ -256,6 +260,22 @@ TEST_CASE_METHOD(
   check_read(qc, [](const QCSetsCell& c) { return !(c.a2 == "wilma"); });
 }
 
+TEST_CASE_METHOD(
+    CPPQueryConditionFx,
+    "IN - String With Non-Enumeration Value",
+    "[query-condition][set][non-enum-value][string]") {
+  auto type = GENERATE(
+      TestArrayType::DENSE, TestArrayType::SPARSE, TestArrayType::LEGACY);
+  auto serialize = SERIALIZE_TESTS();
+  create_array(type, serialize);
+
+  std::vector<std::string> values = {"wilma", "astro"};
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr2", values, TILEDB_NOT_IN);
+
+  check_read(qc, [](const QCSetsCell& c) { return !(c.a2 == "wilma"); });
+}
+
 TEST_CASE_METHOD(
     CPPQueryConditionFx,
     "NOT_IN - Enumeration",
@@ -348,7 +368,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"wilma"};
-  auto qc1 = QueryConditionExperimental::create(ctx_, "attr2", values);
+  auto qc1 =
+      QueryConditionExperimental::create(ctx_, "attr2", values, TILEDB_IN);
   auto qc2 = qc1.negate();
 
   check_read(qc2, [](const QCSetsCell& c) { return !(c.a2 == "wilma"); });
@@ -383,7 +404,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"wilma", "betty"};
-  auto qc1 = QueryConditionExperimental::create(ctx_, "attr2", values);
+  auto qc1 =
+      QueryConditionExperimental::create(ctx_, "attr2", values, TILEDB_IN);
   auto qc2 = QueryCondition::create(ctx_, "attr1", 2.0f, TILEDB_GT);
   auto qc3 = qc1.combine(qc2, TILEDB_AND);
 
@@ -402,7 +424,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"wilma", "betty"};
-  auto qc1 = QueryConditionExperimental::create(ctx_, "attr2", values);
+  auto qc1 =
+      QueryConditionExperimental::create(ctx_, "attr2", values, TILEDB_IN);
   auto qc2 = QueryCondition::create(ctx_, "attr1", 3.0f, TILEDB_EQ);
   auto qc3 = qc1.combine(qc2, TILEDB_OR);
 
@@ -420,11 +443,13 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> del_values = {"wilma"};
-  auto del_qc = QueryConditionExperimental::create(ctx_, "attr2", del_values);
+  auto del_qc =
+      QueryConditionExperimental::create(ctx_, "attr2", del_values, TILEDB_IN);
   write_delete(del_qc);
 
   std::vector<std::string> values = {"wilma", "betty"};
-  auto qc = QueryConditionExperimental::create(ctx_, "attr2", values);
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr2", values, TILEDB_IN);
 
   check_read(qc, [](const QCSetsCell& c) {
     // Every instance of "wilma" was deleted so we only expect "betty"
@@ -442,7 +467,7 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"", "foo"};
-  auto qc = QueryConditionExperimental::create(ctx_, "dim", values);
+  auto qc = QueryConditionExperimental::create(ctx_, "dim", values, TILEDB_IN);
 
   REQUIRE_THROWS(check_read(qc, [](const QCSetsCell&) -> bool {
     throw std::logic_error("Shouldn't get here.");
@@ -459,7 +484,8 @@ TEST_CASE_METHOD(
   create_array(type, serialize);
 
   std::vector<std::string> values = {"oh", "hi"};
-  auto qc = QueryConditionExperimental::create(ctx_, "attr5", values);
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr5", values, TILEDB_IN);
 
   REQUIRE_THROWS(check_read(qc, [](const QCSetsCell&) -> bool {
     throw std::logic_error("Shouldn't get here.");
@@ -818,7 +844,7 @@ void CPPQueryConditionFx::rm_array() {
 }
 
 void CPPQueryConditionFx::generate_data() {
-  num_elements_ = 10;  // * 1024;
+  num_elements_ = 1024;
 
   dim_values_.clear();
   attr1_values_.clear();
@@ -914,7 +940,7 @@ QueryCondition CPPQueryConditionFx::serialize_deserialize_qc(
   throw_if_not_ok(condition_to_capnp(*qc_ptr, &builder));
 
   // Deserialize the query condition.
-  throw_if_not_ok(condition_from_capnp(builder, ret_ptr));
+  *ret_ptr = condition_from_capnp(builder);
   REQUIRE(tiledb::test::ast_equal(ret_ptr->ast(), qc_ptr->ast()));
 
   return ret;
@@ -1002,10 +1028,11 @@ std::vector<std::string> CPPQueryConditionFx::to_vector(
 template <typename T>
 T CPPQueryConditionFx::choose_value(std::vector<T>& values) {
   auto rval = random();
-  auto idx = static_cast<size_t>(rval * values.size());
+  // Note the `% values.size()` which handles when rval is 1.0
+  auto idx = static_cast<size_t>(rval * values.size()) % values.size();
   return values[idx];
 }
 
 float CPPQueryConditionFx::random() {
-  return static_cast<float>(std::rand()) / RAND_MAX;
+  return static_cast<float>(std::rand()) / static_cast<float>(RAND_MAX);
 }
