@@ -45,6 +45,7 @@
 #include "tiledb/api/c_api/context/context_api_external.h"
 #include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
@@ -56,24 +57,33 @@
 #include "tiledb/sm/serialization/array.h"
 #include "tiledb/sm/serialization/query.h"
 
+int setenv_local(const char* __name, const char* __value) {
+#ifdef _WIN32
+  return _putenv_s(__name, __value);
+#else
+  return ::setenv(__name, __value, 1);
+#endif
+}
+
 std::mutex catch2_macro_mutex;
 
-namespace tiledb {
-namespace test {
-
-const std::string& get_temp_path() {
-  // Ensure the path has a trailing delimiter.
-  static std::string temp_path =
-      (std::filesystem::temp_directory_path() /
-       ("tiledb_test_" + std::to_string(getpid())) / "")
-          .string();
-
-  return temp_path;
-}
+namespace tiledb::test {
 
 // Command line arguments.
 std::string g_vfs;
 
+void throw_if_setup_failed(capi_return_t rc) {
+  if (rc != TILEDB_OK) {
+    throw std::runtime_error("Test setup failed.");
+  }
+}
+
+void throw_if_setup_failed(bool condition) {
+  if (!condition) {
+    throw std::runtime_error("Test setup failed.");
+  }
+}
+
 void check_tiledb_error_with(
     tiledb_ctx_t* ctx, int rc, const std::string& expected_msg, bool contains) {
   CHECK(rc == TILEDB_ERR);
@@ -558,7 +568,6 @@ void create_array(
     const std::string& array_name,
     tiledb_encryption_type_t enc_type,
     const char* key,
-    uint32_t key_len,
     tiledb_array_type_t array_type,
     const std::vector<std::string>& dim_names,
     const std::vector<tiledb_datatype_t>& dim_types,
@@ -651,8 +660,6 @@ void create_array(
   rc = tiledb_config_set(config, "sm.encryption_key", key, &error);
   REQUIRE(rc == TILEDB_OK);
   REQUIRE(error == nullptr);
-  tiledb::sm::UnitTestConfig::instance().array_encryption_key_length.set(
-      key_len);
   tiledb_ctx_t* ctx_array;
   REQUIRE(tiledb_ctx_alloc(config, &ctx_array) == TILEDB_OK);
   rc = tiledb_array_create(ctx_array, array_name.c_str(), array_schema);
@@ -785,63 +792,59 @@ void create_azure_container(
   }
 }
 
-void create_ctx_and_vfs(
-    bool s3_supported,
-    bool azure_supported,
-    tiledb_ctx_t** ctx,
-    tiledb_vfs_t** vfs) {
-  // Create TileDB context
-  tiledb_config_t* config = nullptr;
-  tiledb_error_t* error = nullptr;
-  REQUIRE(tiledb_config_alloc(&config, &error) == TILEDB_OK);
-  REQUIRE(error == nullptr);
-  if (s3_supported) {
-#ifndef TILEDB_TESTS_AWS_S3_CONFIG
-    REQUIRE(
-        tiledb_config_set(
-            config, "vfs.s3.endpoint_override", "localhost:9999", &error) ==
-        TILEDB_OK);
-    REQUIRE(
-        tiledb_config_set(config, "vfs.s3.scheme", "https", &error) ==
-        TILEDB_OK);
-    REQUIRE(
-        tiledb_config_set(
-            config, "vfs.s3.use_virtual_addressing", "false", &error) ==
-        TILEDB_OK);
-    REQUIRE(
-        tiledb_config_set(config, "vfs.s3.verify_ssl", "false", &error) ==
-        TILEDB_OK);
-    REQUIRE(error == nullptr);
-#endif
-  }
-  if (azure_supported) {
-    REQUIRE(
-        tiledb_config_set(
-            config,
-            "vfs.azure.storage_account_name",
-            "devstoreaccount1",
-            &error) == TILEDB_OK);
-    REQUIRE(
-        tiledb_config_set(
-            config,
-            "vfs.azure.storage_account_key",
-            "Eby8vdM02xNOcqFlqUwJPLlmEtlCDXJ1OUzFT50uSRZ6IFsuFq2UVErCz4I6tq/"
-            "K1SZFPTOtr/KBHBeksoGMGw==",
-            &error) == TILEDB_OK);
-    REQUIRE(
-        tiledb_config_set(
-            config,
-            "vfs.azure.blob_endpoint",
-            "http://127.0.0.1:10000/devstoreaccount1",
-            &error) == TILEDB_OK);
-  }
-  REQUIRE(tiledb_ctx_alloc(config, ctx) == TILEDB_OK);
-  REQUIRE(error == nullptr);
+void get_supported_fs(
+    bool* s3_supported,
+    bool* hdfs_supported,
+    bool* azure_supported,
+    bool* gcs_supported) {
+  // Override VFS support if the user used the '--vfs' command line argument.
+  if (g_vfs.empty()) {
+    *s3_supported = tiledb::sm::filesystem::s3_enabled;
+    *hdfs_supported = tiledb::sm::filesystem::hdfs_enabled;
+    *azure_supported = tiledb::sm::filesystem::azure_enabled;
+    *gcs_supported = tiledb::sm::filesystem::gcs_enabled;
+  } else {
+    if (!(g_vfs == "native" || g_vfs == "s3" || g_vfs == "hdfs" ||
+          g_vfs == "azure" || g_vfs == "gcs")) {
+      throw std::runtime_error(
+          "Failed to get supported fs. Invalid --vfs command line argument.");
+    }
 
-  // Create VFS
-  *vfs = nullptr;
-  REQUIRE(tiledb_vfs_alloc(*ctx, config, vfs) == TILEDB_OK);
-  tiledb_config_free(&config);
+    if (g_vfs == "native") {
+      *s3_supported = false;
+      *hdfs_supported = false;
+      *azure_supported = false;
+      *gcs_supported = false;
+    }
+
+    if (g_vfs == "s3") {
+      *s3_supported = true;
+      *hdfs_supported = false;
+      *azure_supported = false;
+      *gcs_supported = false;
+    }
+
+    if (g_vfs == "hdfs") {
+      *s3_supported = false;
+      *hdfs_supported = true;
+      *azure_supported = false;
+      *gcs_supported = false;
+    }
+
+    if (g_vfs == "azure") {
+      *s3_supported = false;
+      *hdfs_supported = false;
+      *azure_supported = true;
+      *gcs_supported = false;
+    }
+
+    if (g_vfs == "gcs") {
+      *s3_supported = false;
+      *hdfs_supported = false;
+      *azure_supported = false;
+      *gcs_supported = true;
+    }
+  }
 }
 
 void create_dir(const std::string& path, tiledb_ctx_t* ctx, tiledb_vfs_t* vfs) {
@@ -926,86 +929,12 @@ void create_subarray(
   *returned_subarray = psubarray;
 }
 
-void get_supported_fs(
-    bool* s3_supported,
-    bool* hdfs_supported,
-    bool* azure_supported,
-    bool* gcs_supported) {
-  tiledb_ctx_t* ctx = nullptr;
-  REQUIRE(tiledb_ctx_alloc(nullptr, &ctx) == TILEDB_OK);
-
-  int is_supported = 0;
-  int rc = tiledb_ctx_is_supported_fs(ctx, TILEDB_S3, &is_supported);
-  REQUIRE(rc == TILEDB_OK);
-  *s3_supported = (bool)is_supported;
-  rc = tiledb_ctx_is_supported_fs(ctx, TILEDB_HDFS, &is_supported);
-  REQUIRE(rc == TILEDB_OK);
-  *hdfs_supported = (bool)is_supported;
-  rc = tiledb_ctx_is_supported_fs(ctx, TILEDB_AZURE, &is_supported);
-  REQUIRE(rc == TILEDB_OK);
-  *azure_supported = (bool)is_supported;
-  rc = tiledb_ctx_is_supported_fs(ctx, TILEDB_GCS, &is_supported);
-  REQUIRE(rc == TILEDB_OK);
-  *gcs_supported = (bool)is_supported;
-
-  // Override VFS support if the user used the '--vfs' command line argument.
-  if (!g_vfs.empty()) {
-    REQUIRE(
-        (g_vfs == "native" || g_vfs == "s3" || g_vfs == "hdfs" ||
-         g_vfs == "azure" || g_vfs == "gcs"));
-
-    if (g_vfs == "native") {
-      *s3_supported = false;
-      *hdfs_supported = false;
-      *azure_supported = false;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "s3") {
-      *s3_supported = true;
-      *hdfs_supported = false;
-      *azure_supported = false;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "hdfs") {
-      *s3_supported = false;
-      *hdfs_supported = true;
-      *azure_supported = false;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "azure") {
-      *s3_supported = false;
-      *hdfs_supported = false;
-      *azure_supported = true;
-      *gcs_supported = false;
-    }
-
-    if (g_vfs == "gcs") {
-      *s3_supported = false;
-      *hdfs_supported = false;
-      *azure_supported = false;
-      *gcs_supported = true;
-    }
-  }
-
-  tiledb_ctx_free(&ctx);
-}
-
 void open_array(
     tiledb_ctx_t* ctx, tiledb_array_t* array, tiledb_query_type_t query_type) {
   int rc = tiledb_array_open(ctx, array, query_type);
   CHECK(rc == TILEDB_OK);
 }
 
-std::string random_name(const std::string& prefix) {
-  std::stringstream ss;
-  ss << prefix << "-" << std::this_thread::get_id() << "-"
-     << TILEDB_TIMESTAMP_NOW_MS;
-  return ss.str();
-}
-
 void remove_dir(const std::string& path, tiledb_ctx_t* ctx, tiledb_vfs_t* vfs) {
   int is_dir = 0;
   REQUIRE(tiledb_vfs_is_dir(ctx, vfs, path.c_str(), &is_dir) == TILEDB_OK);
@@ -1079,7 +1008,6 @@ void write_array(
     const std::string& array_name,
     tiledb_encryption_type_t encryption_type,
     const char* key,
-    uint64_t key_len,
     uint64_t timestamp,
     tiledb_layout_t layout,
     const QueryBuffers& buffers) {
@@ -1088,7 +1016,6 @@ void write_array(
       array_name,
       encryption_type,
       key,
-      key_len,
       timestamp,
       nullptr,
       layout,
@@ -1122,7 +1049,6 @@ void write_array(
     const std::string& array_name,
     tiledb_encryption_type_t encryption_type,
     const char* key,
-    uint64_t key_len,
     uint64_t timestamp,
     const void* subarray,
     tiledb_layout_t layout,
@@ -1133,7 +1059,6 @@ void write_array(
       array_name,
       encryption_type,
       key,
-      key_len,
       timestamp,
       subarray,
       layout,
@@ -1157,7 +1082,6 @@ void write_array(
     const std::string& array_name,
     tiledb_encryption_type_t encryption_type,
     const char* key,
-    uint64_t key_len,
     uint64_t timestamp,
     tiledb_layout_t layout,
     const QueryBuffers& buffers,
@@ -1167,7 +1091,6 @@ void write_array(
       array_name,
       encryption_type,
       key,
-      key_len,
       timestamp,
       nullptr,
       layout,
@@ -1187,8 +1110,7 @@ void write_array(
       ctx,
       array_name,
       TILEDB_NO_ENCRYPTION,
-      nullptr,
-      0,
+      "",
       timestamp,
       subarray,
       layout,
@@ -1201,7 +1123,6 @@ void write_array(
     const std::string& array_name,
     tiledb_encryption_type_t encryption_type,
     const char* key,
-    uint64_t key_len,
     uint64_t timestamp,
     const void* sub,
     tiledb_layout_t layout,
@@ -1232,8 +1153,6 @@ void write_array(
     REQUIRE(err == nullptr);
     rc = tiledb_array_set_config(ctx, array, cfg);
     REQUIRE(rc == TILEDB_OK);
-    tiledb::sm::UnitTestConfig::instance().array_encryption_key_length.set(
-        key_len);
   }
   rc = tiledb_array_open(ctx, array, TILEDB_WRITE);
   CHECK(rc == TILEDB_OK);
@@ -1682,7 +1601,8 @@ int array_open_wrapper(
       (*open_array)->array_.get(),
       tiledb::sm::SerializationType::CAPNP,
       buff->buffer(),
-      client_ctx->storage_manager());
+      client_ctx->storage_manager(),
+      tiledb::test::create_test_memory_tracker());
   REQUIRE(st.ok());
 
   // 6. Server: Close array and clean up
@@ -2397,6 +2317,4 @@ template void check_counts<int32_t>(
 template void check_counts<uint64_t>(
     span<uint64_t> vals, std::vector<uint64_t> expected);
 
-}  // End of namespace test
-
-}  // End of namespace tiledb
+}  // namespace tiledb::test
