@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2022-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2022-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,12 +36,12 @@
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/query_status.h"
 #include "tiledb/sm/enums/query_type.h"
+#include "tiledb/sm/fragment/fragment_identifier.h"
 #include "tiledb/sm/misc/tdb_time.h"
 #include "tiledb/sm/query/query.h"
 #include "tiledb/sm/stats/global_stats.h"
 #include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/storage_format/uri/generate_uri.h"
-#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <iostream>
 #include <numeric>
@@ -184,7 +184,7 @@ FragmentConsolidator::FragmentConsolidator(
     : Consolidator(storage_manager) {
   auto st = set_config(config);
   if (!st.ok()) {
-    throw std::logic_error(st.message());
+    throw FragmentConsolidatorException(st.message());
   }
 }
 
@@ -518,9 +518,8 @@ Status FragmentConsolidator::consolidate_internal(
   if (!config_.purge_deleted_cells_ &&
       array_schema.write_version() >= constants::deletes_min_version) {
     // Get the first fragment first timestamp.
-    std::pair<uint64_t, uint64_t> timestamps;
-    RETURN_NOT_OK(
-        utils::parse::get_timestamp_range(to_consolidate[0].uri_, &timestamps));
+    FragmentID fragment_id{to_consolidate[0].uri_};
+    auto timestamps{fragment_id.timestamp_range()};
 
     for (auto& delete_and_update_tile_location :
          array_for_reads->array_directory()
@@ -842,7 +841,7 @@ void FragmentConsolidator::set_query_buffers(
   const auto& array_schema = query->array_schema();
   auto dim_num = array_schema.dim_num();
   auto dense = array_schema.dense();
-  auto attributes = array_schema.attributes();
+  auto& attributes = array_schema.attributes();
   unsigned bid = 0;
 
   // Here the first buffer should always be the fixed buffer (either offsets
