@@ -553,7 +553,7 @@ class S3Scanner : public LsScanner<F, D> {
  * This class implements the various S3 filesystem functions. It also
  * maintains buffer caches for writing into the various attribute files.
  */
-class S3 {
+class S3 : FilesystemBase {
  private:
   /** Forward declaration */
   struct MultiPartUploadState;
@@ -586,9 +586,213 @@ class S3 {
    * Creates a bucket.
    *
    * @param bucket The name of the bucket to be created.
-   * @return Status
    */
-  Status create_bucket(const URI& bucket) const;
+  void create_bucket(const URI& bucket) const override;
+
+  /**
+   * Removes the contents of an S3 bucket.
+   *
+   * @param bucket The URI of the bucket to be emptied.
+   */
+  void empty_bucket(const URI& bucket) const override;
+
+  /**
+   * Checks if a bucket is empty.
+   *
+   * @param bucket The URI of the bucket.
+   * @return True if the bucket is empty, false otherwise.
+   */
+  bool is_empty_bucket(const URI& bucket) const override;
+
+  /**
+   * Check if a bucket exists.
+   *
+   * @param bucket The name of the bucket.
+   * @return True if the bucket exists, false otherwise.
+   */
+  bool is_bucket(const URI& uri) const override;
+
+  /**
+   * Renames a directory. Note that this is an expensive operation.
+   * The function will essentially copy all objects with directory
+   * prefix `old_uri` to new objects with prefix `new_uri` and then
+   * delete the old ones.
+   *
+   * @param old_uri The URI of the old path.
+   * @param new_uri The URI of the new path.
+   */
+  void move_dir(const URI& old_uri, const URI& new_uri) const override;
+
+  /**
+   * Copies a file.
+   *
+   * @param old_uri The URI of the old path.
+   * @param new_uri The URI of the new path.
+   */
+  void copy_file(const URI& old_uri, const URI& new_uri) const override;
+
+  /**
+   * Copies a directory. All subdirectories and files are copied.
+   *
+   * @param old_uri The URI of the old path.
+   * @param new_uri The URI of the new path.
+   */
+  void copy_dir(const URI& old_uri, const URI& new_uri) const override;
+
+  /**
+   * Reads from a file.
+   *
+   * @param uri The URI of the file.
+   * @param offset The offset where the read begins.
+   * @param buffer The buffer to read into.
+   * @param nbytes Number of bytes to read.
+   * @param use_read_ahead Whether to use the read-ahead cache.
+   */
+  void read(
+      const URI& uri,
+      uint64_t offset,
+      void* buffer,
+      uint64_t nbytes,
+      bool use_read_ahead = true) const override;
+
+  /**
+   * Deletes a bucket.
+   *
+   * @param bucket The name of the bucket to be deleted.
+   */
+  void remove_bucket(const URI& bucket) const override;
+
+  /**
+   * Deletes all objects with prefix `prefix/` (if the ending `/` does not
+   * exist in `prefix`, it is added by the function.
+   *
+   * For instance, suppose there exist the following objects:
+   * - `s3://some_bucket/foo/bar1`
+   * - `s3://some_bucket/foo/bar2/bar3
+   * - `s3://some_bucket/foo/bar4
+   * - `s3://some_bucket/foo2`
+   *
+   * `remove("s3://some_bucket/foo")` and `remove("s3://some_bucket/foo/")`
+   * will delete objects:
+   *
+   * - `s3://some_bucket/foo/bar1`
+   * - `s3://some_bucket/foo/bar2/bar3
+   * - `s3://some_bucket/foo/bar4
+   *
+   * In contrast, `remove("s3://some_bucket/foo2")` will not delete anything,
+   * the function internally appends `/` to the end of the URI, and therefore
+   * there is not object with prefix "s3://some_bucket/foo2/" in this example.
+   *
+   * @param prefix The prefix of the objects to be deleted.
+   */
+  void remove_dir(const URI& prefix) const override;
+
+  /**
+   * Creates an empty object.
+   *
+   * @param uri The URI of the object to be created.
+   */
+  void touch(const URI& uri) const override;
+
+  /**
+   * Writes the input buffer to an S3 object. Note that this is essentially
+   * an append operation implemented via multipart uploads.
+   *
+   * @param uri The URI of the object to be written to.
+   * @param buffer The input buffer.
+   * @param length The size of the input buffer.
+   * @param remote_global_order_write
+   */
+  void write(
+      const URI& uri,
+      const void* buffer,
+      uint64_t length,
+      bool remote_global_order_write = false) override;
+
+  /**
+   * Creates a directory.
+   *
+   * - On S3, this is a noop.
+   * - On all other backends, if the directory exists, the function
+   *   just succeeds without doing anything.
+   *
+   * @param uri The URI of the directory.
+   */
+  void create_dir(const URI&) const override {
+    // No-op for S3.
+  }
+
+  /**
+   * Checks if a file exists.
+   *
+   * @param uri The URI to check for existence.
+   * @return True if the file exists, else False.
+   */
+  bool is_file(const URI& uri) const override {
+    bool object = false;
+    throw_if_not_ok(is_object(uri, &object));
+    return object;
+  }
+
+  /**
+   * Deletes a file.
+   *
+   * @param uri The URI of the file.
+   */
+  void remove_file(const URI& uri) const override {
+    throw_if_not_ok(remove_object(uri));
+  }
+
+  /**
+   * Retrieves the size of a file.
+   *
+   * @param uri The URI of the file.
+   * @param size The file size to be retrieved.
+   */
+  void file_size(const URI& uri, uint64_t* size) const override {
+    throw_if_not_ok(object_size(uri, size));
+  }
+
+  /**
+   * Renames a file.
+   * Both URI must be of the same backend type. (e.g. both s3://, file://, etc)
+   *
+   * @param old_uri The old URI.
+   * @param new_uri The new URI.
+   */
+  void move_file(const URI& old_uri, const URI& new_uri) const override {
+    throw_if_not_ok(move_object(old_uri, new_uri));
+  }
+
+  /**
+   * Syncs (flushes) a file. Note that for S3 this is a noop.
+   *
+   * @param uri The URI of the file.
+   */
+  void sync(const URI&) const override {
+    // No-op for S3.
+  }
+
+  /**
+   * Checks if a directory exists.
+   *
+   * @param uri The URI to check for existence.
+   * @return True if the directory exists, else False.
+   */
+  bool is_dir(const URI& uri) const override {
+    bool dir = false;
+    throw_if_not_ok(is_dir(uri, &dir));
+    return dir;
+  }
+
+  /**
+   * Retrieves all the entries contained in the parent.
+   *
+   * @param parent The target directory to list.
+   * @return All entries that are contained in the parent
+   */
+  tuple<Status, optional<std::vector<directory_entry>>> ls_with_sizes(
+      const URI& parent) const override;
 
   /**
    * Disconnects a S3 client.
@@ -596,10 +800,6 @@ class S3 {
    * @return Status
    */
   Status disconnect();
-
-  /** Removes the contents of an S3 bucket. */
-  Status empty_bucket(const URI& bucket) const;
-
   /**
    * Flushes an object to S3, finalizing the multipart upload.
    *
@@ -616,19 +816,6 @@ class S3 {
    */
   void finalize_and_flush_object(const URI& uri);
 
-  /** Checks if a bucket is empty. */
-  Status is_empty_bucket(const URI& bucket, bool* is_empty) const;
-
-  /**
-   * Check if a bucket exists.
-   *
-   * @param bucket The name of the bucket.
-   * @param exists Mutates to `true` if `uri` is an existing bucket,
-   *   and `false` otherwise.
-   * @return Status
-   */
-  Status is_bucket(const URI& uri, bool* exists) const;
-
   /**
    * Checks if there is an object with prefix `uri/`. For instance, suppose
    * the following objects exist:
@@ -659,6 +846,12 @@ class S3 {
    */
   Status is_object(const URI& uri, bool* exists) const;
 
+  /** Checks if the given object exists on S3. */
+  Status is_object(
+      const Aws::String& bucket_name,
+      const Aws::String& object_key,
+      bool* exists) const;
+
   /**
    * Lists the objects that start with `prefix`. Full URI paths are
    * retrieved for the matched objects. If a delimiter is specified,
@@ -699,9 +892,7 @@ class S3 {
    * @return Status tuple where second is a list of directory_entry objects.
    */
   tuple<Status, optional<std::vector<directory_entry>>> ls_with_sizes(
-      const URI& prefix,
-      const std::string& delimiter = "/",
-      int max_paths = -1) const;
+      const URI& prefix, const std::string& delimiter, int max_paths) const;
 
   /**
    * Lists objects and object information that start with `prefix`, invoking
@@ -764,37 +955,7 @@ class S3 {
    * @param new_uri The URI of the new path.
    * @return Status
    */
-  Status move_object(const URI& old_uri, const URI& new_uri);
-
-  /**
-   * Renames a directory. Note that this is an expensive operation.
-   * The function will essentially copy all objects with directory
-   * prefix `old_uri` to new objects with prefix `new_uri` and then
-   * delete the old ones.
-   *
-   * @param old_uri The URI of the old path.
-   * @param new_uri The URI of the new path.
-   * @return Status
-   */
-  Status move_dir(const URI& old_uri, const URI& new_uri);
-
-  /**
-   * Copies a file.
-   *
-   * @param old_uri The URI of the old path.
-   * @param new_uri The URI of the new path.
-   * @return Status
-   */
-  Status copy_file(const URI& old_uri, const URI& new_uri);
-
-  /**
-   * Copies a directory. All subdirectories and files are copied.
-   *
-   * @param old_uri The URI of the old path.
-   * @param new_uri The URI of the new path.
-   * @return Status
-   */
-  Status copy_dir(const URI& old_uri, const URI& new_uri);
+  Status move_object(const URI& old_uri, const URI& new_uri) const;
 
   /**
    * Returns the size of the input object with a given URI in bytes.
@@ -816,21 +977,13 @@ class S3 {
    * @param length_returned Returns the total length read into `buffer`.
    * @return Status
    */
-  Status read(
+  Status read_impl(
       const URI& uri,
-      off_t offset,
-      void* buffer,
-      uint64_t length,
-      uint64_t read_ahead_length,
-      uint64_t* length_returned) const;
-
-  /**
-   * Deletes a bucket.
-   *
-   * @param bucket The name of the bucket to be deleted.
-   * @return Status
-   */
-  Status remove_bucket(const URI& bucket) const;
+      const off_t offset,
+      void* const buffer,
+      const uint64_t length,
+      const uint64_t read_ahead_length,
+      uint64_t* const length_returned) const;
 
   /**
    * Deletes an object with a given URI.
@@ -840,51 +993,6 @@ class S3 {
    */
   Status remove_object(const URI& uri) const;
 
-  /**
-   * Deletes all objects with prefix `prefix/` (if the ending `/` does not
-   * exist in `prefix`, it is added by the function.
-   *
-   * For instance, suppose there exist the following objects:
-   * - `s3://some_bucket/foo/bar1`
-   * - `s3://some_bucket/foo/bar2/bar3
-   * - `s3://some_bucket/foo/bar4
-   * - `s3://some_bucket/foo2`
-   *
-   * `remove("s3://some_bucket/foo")` and `remove("s3://some_bucket/foo/")`
-   * will delete objects:
-   *
-   * - `s3://some_bucket/foo/bar1`
-   * - `s3://some_bucket/foo/bar2/bar3
-   * - `s3://some_bucket/foo/bar4
-   *
-   * In contrast, `remove("s3://some_bucket/foo2")` will not delete anything,
-   * the function internally appends `/` to the end of the URI, and therefore
-   * there is not object with prefix "s3://some_bucket/foo2/" in this example.
-   *
-   * @param uri The prefix of the objects to be deleted.
-   * @return Status
-   */
-  Status remove_dir(const URI& prefix) const;
-
-  /**
-   * Creates an empty object.
-   *
-   * @param uri The URI of the object to be created.
-   * @return Status
-   */
-  Status touch(const URI& uri) const;
-
-  /**
-   * Writes the input buffer to an S3 object. Note that this is essentially
-   * an append operation implemented via multipart uploads.
-   *
-   * @param uri The URI of the object to be written to.
-   * @param buffer The input buffer.
-   * @param length The size of the input buffer.
-   * @return Status
-   */
-  Status write(const URI& uri, const void* buffer, uint64_t length);
-
   /**
    * Writes the input buffer to an S3 object. This function buffers in memory
    * file_buffer_size_ bytes before calling global_order_write() to execute
@@ -1308,7 +1416,7 @@ class S3 {
    * @param new_uri The newly created object.
    * @return Status
    */
-  Status copy_object(const URI& old_uri, const URI& new_uri);
+  Status copy_object(const URI& old_uri, const URI& new_uri) const;
 
   /**
    * Fills the file buffer (given as an input `Buffer` object) from the
@@ -1363,12 +1471,6 @@ class S3 {
   std::string join_authority_and_path(
       const std::string& authority, const std::string& path) const;
 
-  /** Checks if the given object exists on S3. */
-  Status is_object(
-      const Aws::String& bucket_name,
-      const Aws::String& object_key,
-      bool* exists) const;
-
   /** Waits for the input object to be propagated. */
   Status wait_for_object_to_propagate(
       const Aws::String& bucketName, const Aws::String& objectKey) const;
