@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,19 +33,18 @@
 #include "tiledb/sm/metadata/metadata.h"
 #include "tiledb/common/exception/exception.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/misc/tdb_time.h"
-#include "tiledb/sm/misc/uuid.h"
-#include "tiledb/storage_format/serialization/serializers.h"
+#include "tiledb/storage_format/uri/generate_uri.h"
 
 #include <iostream>
 #include <sstream>
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class MetadataException : public StatusException {
  public:
@@ -57,44 +56,52 @@ class MetadataException : public StatusException {
 /* ********************************* */
 /*     CONSTRUCTORS & DESTRUCTORS    */
 /* ********************************* */
-
-Metadata::Metadata()
-    : Metadata(std::map<std::string, MetadataValue>()) {
-}
-
-Metadata::Metadata(const std::map<std::string, MetadataValue>& metadata_map)
-    : metadata_map_(metadata_map)
+Metadata::Metadata(shared_ptr<MemoryTracker> memory_tracker)
+    : memory_tracker_(memory_tracker)
+    , metadata_map_(memory_tracker_->get_resource(MemoryType::METADATA))
+    , metadata_index_(memory_tracker_->get_resource(MemoryType::METADATA))
     , timestamp_range_([]() -> std::pair<uint64_t, uint64_t> {
       auto t = utils::time::timestamp_now_ms();
       return std::make_pair(t, t);
-    }()) {
+    }())
+    , loaded_metadata_uris_(
+          memory_tracker_->get_resource(MemoryType::METADATA)) {
   build_metadata_index();
 }
 
-Metadata::Metadata(const Metadata& rhs)
-    : metadata_map_(rhs.metadata_map_)
-    , timestamp_range_(rhs.timestamp_range_)
-    , loaded_metadata_uris_(rhs.loaded_metadata_uris_)
-    , uri_(rhs.uri_) {
-  if (!rhs.metadata_index_.empty())
-    build_metadata_index();
-}
-
-Metadata& Metadata::operator=(const Metadata& other) {
-  metadata_map_ = other.metadata_map_;
-  timestamp_range_ = other.timestamp_range_;
-  loaded_metadata_uris_ = other.loaded_metadata_uris_;
-  uri_ = other.uri_;
-  build_metadata_index();
-  return *this;
-}
-
-Metadata::~Metadata() = default;
-
 /* ********************************* */
 /*                API                */
 /* ********************************* */
 
+Metadata& Metadata::operator=(Metadata& other) {
+  clear();
+  for (auto& [k, v] : other.metadata_map_) {
+    metadata_map_.emplace(k, v);
+  }
+
+  timestamp_range_ = other.timestamp_range_;
+
+  for (auto& uri : other.loaded_metadata_uris_) {
+    loaded_metadata_uris_.emplace_back(uri);
+  }
+
+  build_metadata_index();
+
+  return *this;
+}
+
+Metadata& Metadata::operator=(
+    std::map<std::string, Metadata::MetadataValue>&& md_map) {
+  clear();
+  for (auto& [k, v] : md_map) {
+    metadata_map_.emplace(k, v);
+  }
+
+  build_metadata_index();
+
+  return *this;
+}
+
 void Metadata::clear() {
   metadata_map_.clear();
   metadata_index_.clear();
@@ -103,35 +110,27 @@ void Metadata::clear() {
   uri_ = URI();
 }
 
-Status Metadata::get_uri(const URI& array_uri, URI* meta_uri) {
+URI Metadata::get_uri(const URI& array_uri) {
   if (uri_.to_string().empty())
-    RETURN_NOT_OK(generate_uri(array_uri));
+    generate_uri(array_uri);
 
-  *meta_uri = uri_;
-  return Status::Ok();
+  return uri_;
 }
 
-Status Metadata::generate_uri(const URI& array_uri) {
-  std::string uuid;
-  RETURN_NOT_OK(uuid::generate_uuid(&uuid, false));
-
-  std::stringstream ss;
-  ss << "__" << timestamp_range_.first << "_" << timestamp_range_.second << "_"
-     << uuid;
+void Metadata::generate_uri(const URI& array_uri) {
+  auto ts_name = tiledb::storage_format::generate_timestamped_name(
+      timestamp_range_.first, timestamp_range_.second, std::nullopt);
   uri_ = array_uri.join_path(constants::array_metadata_dir_name)
-             .join_path(ss.str());
-
-  return Status::Ok();
+             .join_path(ts_name);
 }
 
-Metadata Metadata::deserialize(
+std::map<std::string, Metadata::MetadataValue> Metadata::deserialize(
     const std::vector<shared_ptr<Tile>>& metadata_tiles) {
   if (metadata_tiles.empty()) {
-    return Metadata();
+    return {};
   }
-  std::map<std::string, MetadataValue> metadata_map;
 
-  Status st;
+  std::map<std::string, MetadataValue> metadata_map;
   uint32_t key_len;
   char del;
   size_t value_len;
@@ -166,7 +165,7 @@ Metadata Metadata::deserialize(
     }
   }
 
-  return Metadata(metadata_map);
+  return metadata_map;
 }
 
 void Metadata::serialize(Serializer& serializer) const {
@@ -322,17 +321,10 @@ void Metadata::set_loaded_metadata_uris(
   timestamp_range_.second = loaded_metadata_uris.back().timestamp_range_.second;
 }
 
-const std::vector<URI>& Metadata::loaded_metadata_uris() const {
+const tdb::pmr::vector<URI>& Metadata::loaded_metadata_uris() const {
   return loaded_metadata_uris_;
 }
 
-void Metadata::swap(Metadata* metadata) {
-  std::swap(metadata_map_, metadata->metadata_map_);
-  std::swap(metadata_index_, metadata->metadata_index_);
-  std::swap(timestamp_range_, metadata->timestamp_range_);
-  std::swap(loaded_metadata_uris_, metadata->loaded_metadata_uris_);
-}
-
 void Metadata::reset(uint64_t timestamp) {
   clear();
   timestamp = (timestamp != 0) ? timestamp : utils::time::timestamp_now_ms();
@@ -365,5 +357,4 @@ void Metadata::build_metadata_index() {
     metadata_index_[i++] = std::make_pair(&(m.first), &(m.second));
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
