@@ -107,52 +107,32 @@ inline IterT skip_invalid_elements(IterT it, const IterT& end) {
 Reader::Reader(
     stats::Stats* stats,
     shared_ptr<Logger> logger,
-    StorageManager* storage_manager,
-    Array* array,
-    Config& config,
-    std::unordered_map<std::string, QueryBuffer>& buffers,
-    std::unordered_map<std::string, QueryBuffer>& aggregate_buffers,
-    Subarray& subarray,
-    Layout layout,
-    std::optional<QueryCondition>& condition,
-    DefaultChannelAggregates& default_channel_aggregates,
-    bool skip_checks_serialization,
+    StrategyParams& params,
     bool remote_query)
-    : ReaderBase(
-          stats,
-          logger->clone("Reader", ++logger_id_),
-          storage_manager,
-          array,
-          config,
-          buffers,
-          aggregate_buffers,
-          subarray,
-          layout,
-          condition,
-          default_channel_aggregates) {
+    : ReaderBase(stats, logger->clone("Reader", ++logger_id_), params) {
   // Sanity checks
   if (storage_manager_ == nullptr) {
     throw ReaderStatusException(
         "Cannot initialize reader; Storage manager not set");
   }
 
-  if (!default_channel_aggregates.empty()) {
+  if (!params.default_channel_aggregates().empty()) {
     throw ReaderStatusException(
         "Cannot initialize reader; Reader cannot process aggregates");
   }
 
-  if (!skip_checks_serialization && buffers_.empty()) {
+  if (!params.skip_checks_serialization() && buffers_.empty()) {
     throw ReaderStatusException("Cannot initialize reader; Buffers not set");
   }
 
-  if (!skip_checks_serialization && array_schema_.dense() &&
+  if (!params.skip_checks_serialization() && array_schema_.dense() &&
       !subarray_.is_set()) {
     throw ReaderStatusException(
         "Cannot initialize reader; Dense reads must have a subarray set");
   }
 
   // Check subarray
-  check_subarray(remote_query && array->array_schema_latest().dense());
+  check_subarray(remote_query && params.array()->array_schema_latest().dense());
 
   // Initialize the read state
   init_read_state();
@@ -473,7 +453,9 @@ Status Reader::compute_range_result_coords(
         result_coords.emplace_back(tile, pos);
     }
   } else {  // Sparse
-    std::vector<uint8_t> result_bitmap(coords_num, 1);
+    auto resource =
+        query_memory_tracker_->get_resource(MemoryType::TILE_BITMAP);
+    tdb::pmr::vector<uint8_t> result_bitmap(coords_num, 1, resource);
 
     // Compute result and overwritten bitmap per dimension
     for (unsigned d = 0; d < dim_num; ++d) {
@@ -512,7 +494,7 @@ Status Reader::compute_range_result_coords(
     Subarray& subarray,
     const std::vector<bool>& single_fragment,
     const std::map<std::pair<unsigned, uint64_t>, size_t>& result_tile_map,
-    std::vector<ResultTile>& result_tiles,
+    IndexedList<ResultTile>& result_tiles,
     std::vector<std::vector<ResultCoords>>& range_result_coords) {
   auto timer_se = stats_->start_timer("compute_range_result_coords");
 
@@ -567,7 +549,7 @@ Status Reader::compute_range_result_coords(
     uint64_t range_idx,
     uint32_t fragment_idx,
     const std::map<std::pair<unsigned, uint64_t>, size_t>& result_tile_map,
-    std::vector<ResultTile>& result_tiles,
+    IndexedList<ResultTile>& result_tiles,
     std::vector<ResultCoords>& range_result_coords) {
   // Skip dense fragments
   if (fragment_metadata_[fragment_idx]->dense())
@@ -588,12 +570,14 @@ Status Reader::compute_range_result_coords(
         auto tile_it = result_tile_map.find(pair);
         assert(tile_it != result_tile_map.end());
         auto tile_idx = tile_it->second;
-        auto& tile = result_tiles[tile_idx];
+
+        auto tile = result_tiles.begin();
+        std::advance(tile, tile_idx);
 
         // Add results only if the sparse tile MBR is not fully
         // covered by a more recent fragment's non-empty domain
         if (!sparse_tile_overwritten(fragment_idx, i))
-          RETURN_NOT_OK(get_all_result_coords(&tile, range_result_coords));
+          RETURN_NOT_OK(get_all_result_coords(&*tile, range_result_coords));
       }
       ++tr;
     } else {
@@ -602,15 +586,16 @@ Status Reader::compute_range_result_coords(
       auto tile_it = result_tile_map.find(pair);
       assert(tile_it != result_tile_map.end());
       auto tile_idx = tile_it->second;
-      auto& tile = result_tiles[tile_idx];
+      auto tile = result_tiles.begin();
+      std::advance(tile, tile_idx);
       if (t->second == 1.0) {  // Full overlap
         // Add results only if the sparse tile MBR is not fully
         // covered by a more recent fragment's non-empty domain
         if (!sparse_tile_overwritten(fragment_idx, t->first))
-          RETURN_NOT_OK(get_all_result_coords(&tile, range_result_coords));
+          RETURN_NOT_OK(get_all_result_coords(&*tile, range_result_coords));
       } else {  // Partial overlap
         RETURN_NOT_OK(compute_range_result_coords(
-            subarray, fragment_idx, &tile, range_idx, range_result_coords));
+            subarray, fragment_idx, &*tile, range_idx, range_result_coords));
       }
       ++t;
     }
@@ -623,7 +608,7 @@ Status Reader::compute_range_result_coords(
     Subarray& subarray,
     uint64_t range_idx,
     const std::map<std::pair<unsigned, uint64_t>, size_t>& result_tile_map,
-    std::vector<ResultTile>& result_tiles,
+    IndexedList<ResultTile>& result_tiles,
     std::vector<ResultCoords>& range_result_coords) {
   // Gather result range coordinates per fragment
   auto fragment_num = fragment_metadata_.size();
@@ -698,7 +683,7 @@ Status Reader::compute_subarray_coords(
 }
 
 Status Reader::compute_sparse_result_tiles(
-    std::vector<ResultTile>& result_tiles,
+    IndexedList<ResultTile>& result_tiles,
     std::map<std::pair<unsigned, uint64_t>, size_t>* result_tile_map,
     std::vector<bool>* single_fragment) {
   auto timer_se = stats_->start_timer("compute_sparse_result_tiles");
@@ -731,7 +716,8 @@ Status Reader::compute_sparse_result_tiles(
           auto pair = std::pair<unsigned, uint64_t>(f, t);
           // Add tile only if it does not already exist
           if (result_tile_map->find(pair) == result_tile_map->end()) {
-            result_tiles.emplace_back(f, t, *fragment_metadata_[f].get());
+            result_tiles.emplace_back(
+                f, t, *fragment_metadata_[f].get(), query_memory_tracker_);
             (*result_tile_map)[pair] = result_tiles.size() - 1;
           }
           // Always check range for multiple fragments or fragments with
@@ -750,7 +736,8 @@ Status Reader::compute_sparse_result_tiles(
         auto pair = std::pair<unsigned, uint64_t>(f, t);
         // Add tile only if it does not already exist
         if (result_tile_map->find(pair) == result_tile_map->end()) {
-          result_tiles.emplace_back(f, t, *fragment_metadata_[f].get());
+          result_tiles.emplace_back(
+              f, t, *fragment_metadata_[f].get(), query_memory_tracker_);
           (*result_tile_map)[pair] = result_tiles.size() - 1;
         }
         // Always check range for multiple fragments or fragments with
@@ -1600,7 +1587,7 @@ Status Reader::compute_result_cell_slabs_global(
 }
 
 Status Reader::compute_result_coords(
-    std::vector<ResultTile>& result_tiles,
+    IndexedList<ResultTile>& result_tiles,
     std::vector<ResultCoords>& result_coords) {
   auto timer_se = stats_->start_timer("compute_result_coords");
 
@@ -1739,7 +1726,7 @@ Status Reader::dense_read() {
   // `sparse_result_tiles` will hold all the relevant result tiles of
   // sparse fragments
   std::vector<ResultCoords> result_coords;
-  std::vector<ResultTile> sparse_result_tiles;
+  IndexedList<ResultTile> sparse_result_tiles;
   RETURN_NOT_OK(compute_result_coords(sparse_result_tiles, result_coords));
 
   // Compute result cell slabs.
@@ -1800,7 +1787,9 @@ Status Reader::get_all_result_coords(
       array_->timestamp_start(), array_->timestamp_end_opened_at());
   if (fragment_metadata_[tile->frag_idx()]->has_timestamps() &&
       partial_overlap) {
-    std::vector<uint8_t> result_bitmap(coords_num, 1);
+    auto resource =
+        query_memory_tracker_->get_resource(MemoryType::TILE_BITMAP);
+    tdb::pmr::vector<uint8_t> result_bitmap(coords_num, 1, resource);
     RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<uint8_t>(
         *(frag_meta->array_schema().get()), *tile, result_bitmap));
 
@@ -1995,7 +1984,7 @@ Status Reader::sparse_read() {
   // `sparse_result_tiles` will hold all the relevant result tiles of
   // sparse fragments
   std::vector<ResultCoords> result_coords;
-  std::vector<ResultTile> sparse_result_tiles;
+  IndexedList<ResultTile> sparse_result_tiles;
 
   RETURN_NOT_OK(compute_result_coords(sparse_result_tiles, result_coords));
   std::vector<ResultTile*> result_tiles;
@@ -2077,7 +2066,7 @@ bool Reader::sparse_tile_overwritten(
   return false;
 }
 
-void Reader::erase_coord_tiles(std::vector<ResultTile>& result_tiles) const {
+void Reader::erase_coord_tiles(IndexedList<ResultTile>& result_tiles) const {
   for (auto& tile : result_tiles) {
     auto dim_num = array_schema_.dim_num();
     for (unsigned d = 0; d < dim_num; ++d)
