@@ -54,6 +54,7 @@
 #include "tiledb/sm/query/readers/aggregators/query_channel.h"
 #include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/query/validity_vector.h"
+#include "tiledb/sm/rest/rest_client.h"
 #include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/subarray.h"
 
@@ -62,6 +63,14 @@ using namespace tiledb::common;
 namespace tiledb {
 namespace sm {
 
+/** Class for query status exceptions. */
+class QueryStatusException : public StatusException {
+ public:
+  explicit QueryStatusException(const std::string& msg)
+      : StatusException("Query", msg) {
+  }
+};
+
 class Array;
 class ArrayDimensionLabelQueries;
 
@@ -655,6 +664,14 @@ class Query {
   /** Returns the internal stats object. */
   stats::Stats* stats() const;
 
+  /**
+   * Populate the owned stats instance with data.
+   * To be removed when the class will get a C41 constructor.
+   *
+   * @param data Data to populate the stats with.
+   */
+  void set_stats(const stats::StatsData& data);
+
   /** Returns the scratch space used for REST requests. */
   shared_ptr<Buffer> rest_scratch() const;
 
@@ -732,6 +749,21 @@ class Query {
   /** Returns true if the output field is an aggregate. */
   bool is_aggregate(std::string output_field_name) const;
 
+ private:
+  /**
+   * Create the aggregate channel object. This is split out because it's not
+   * at construction time, but on demand, and in two different situations.
+   */
+  void create_aggregate_channel() {
+    /*
+     * Because we have an extremely simple way of choosing channel identifiers,
+     * we can get away with hard-coding `1` here as the identifier for the
+     * aggregate channel.
+     */
+    aggregate_channel_ = make_shared<QueryChannel>(HERE(), *this, 1);
+  }
+
+ public:
   /**
    * Adds an aggregator to the default channel.
    *
@@ -740,6 +772,15 @@ class Query {
    */
   void add_aggregator_to_default_channel(
       std::string output_field_name, shared_ptr<IAggregator> aggregator) {
+    if (default_channel_aggregates_.empty()) {
+      /*
+       * Assert: this is the first aggregate added.
+       *
+       * We create the aggregate channel on demand, and this is when we need to
+       * do it.
+       */
+      create_aggregate_channel();
+    }
     default_channel_aggregates_.emplace(output_field_name, aggregator);
   }
 
@@ -750,20 +791,25 @@ class Query {
   /**
    * Get a list of all channels and their aggregates
    */
-  std::vector<QueryChannel> get_channels() {
+  std::vector<LegacyQueryAggregatesOverDefault> get_channels() {
     // Currently only the default channel is supported
-    return {QueryChannel(true, default_channel_aggregates_)};
+    return {
+        LegacyQueryAggregatesOverDefault(true, default_channel_aggregates_)};
   }
 
   /**
-   * Add a channel to the query
+   * Add a channel to the query. Used only by capnp serialization to initialize
+   * the aggregates list.
    */
-  void add_channel(const QueryChannel& channel) {
+  void add_channel(const LegacyQueryAggregatesOverDefault& channel) {
     if (channel.is_default()) {
       default_channel_aggregates_ = channel.aggregates();
+      if (!default_channel_aggregates_.empty()) {
+        create_aggregate_channel();
+      }
       return;
     }
-    throw std::logic_error(
+    throw QueryStatusException(
         "We currently only support a default channel for queries");
   }
 
@@ -772,22 +818,64 @@ class Query {
    */
   bool has_aggregates() {
     // We only need to check the default channel for now
-    return default_channel_aggregates_.empty();
+    return !default_channel_aggregates_.empty();
   }
 
+  /**
+   * Returns the number of channels.
+   *
+   * Responsibility for choosing channel identifiers is the responsibility of
+   * this class. At the present time the policy is very simple, since all
+   * queries only draw from a single array.
+   *   - Channel 0: All rows from the query. Always non-segmented, that is,
+   *       without any grouping.
+   *   - Channel 1: (optional) Simple aggregates, if any exist.
+   */
+  inline size_t number_of_channels() {
+    return has_aggregates() ? 1 : 0;
+  };
+
+  /**
+   * The default channel is initialized at construction and always exists.
+   */
+  inline std::shared_ptr<QueryChannel> default_channel() {
+    return default_channel_;
+  }
+
+  inline std::shared_ptr<QueryChannel> aggegate_channel() {
+    if (!has_aggregates()) {
+      throw QueryStatusException("Aggregate channel does not exist");
+    }
+    return aggregate_channel_;
+  }
+
+  /**
+   * Returns the REST client configured in the storage manager associated to
+   * this query
+   */
+  RestClient* rest_client() const;
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /** The query memory tracker. */
+  shared_ptr<MemoryTracker> query_memory_tracker_;
+
   /** A smart pointer to the array the query is associated with.
    * Ensures that the Array object exists as long as the Query object exists. */
   shared_ptr<Array> array_shared_;
 
-  /** The array the query is associated with.
-   * Cached copy of array_shared_.get(). */
+  /**
+   * The array the query is associated with.
+   * Cached copy of array_shared_.get().
+   */
   Array* array_;
 
+  /** Keeps a copy of the opened array object at construction. */
+  shared_ptr<OpenedArray> opened_array_;
+
   /** The array schema. */
   shared_ptr<const ArraySchema> array_schema_;
 
@@ -952,6 +1040,25 @@ class Query {
   std::unordered_map<std::string, shared_ptr<IAggregator>>
       default_channel_aggregates_;
 
+  /*
+   * Handles to channels use shared pointers, so the channels are allocated here
+   * for ease of implementation.
+   *
+   * At present there's only one possible non-default channel, so we keep track
+   * of it in its own variable. A fully C.41 class might simply store these in
+   * a constant vector.
+   */
+  /**
+   * The default channel is allocated in the constructor for simplicity.
+   */
+  std::shared_ptr<QueryChannel> default_channel_;
+
+  /**
+   * The aggegregate channel is optional, so we initialize it as empty with it
+   * default constructor.
+   */
+  std::shared_ptr<QueryChannel> aggregate_channel_{};
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
