@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2021-2022 TileDB, Inc.
+ * @copyright Copyright (c) 2021-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,10 +35,10 @@
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/query_condition_combination_op.h"
 #include "tiledb/sm/enums/query_condition_op.h"
+#include "tiledb/sm/fragment/fragment_identifier.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/readers/result_cell_slab.h"
-#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <algorithm>
 #include <functional>
@@ -51,8 +51,7 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 QueryCondition::QueryCondition() {
 }
@@ -127,6 +126,12 @@ Status QueryCondition::init(
     return Status_QueryConditionError("Cannot reinitialize query condition");
   }
 
+  if (op == QueryConditionOp::ALWAYS_TRUE ||
+      op == QueryConditionOp::ALWAYS_FALSE) {
+    auto op_str = query_condition_op_str(op);
+    throw std::runtime_error("Invalid use of internal operation: " + op_str);
+  }
+
   // AST Construction.
   tree_ = tdb_unique_ptr<ASTNode>(tdb_new(
       ASTNodeVal, field_name, condition_value, condition_value_size, op));
@@ -210,12 +215,8 @@ uint64_t QueryCondition::condition_timestamp() const {
     return 0;
   }
 
-  std::pair<uint64_t, uint64_t> timestamps;
-  if (!utils::parse::get_timestamp_range(URI(condition_marker_), &timestamps)
-           .ok()) {
-    throw std::logic_error("Error parsing condition marker.");
-  }
-
+  FragmentID fragment_id{condition_marker_};
+  auto timestamps{fragment_id.timestamp_range()};
   return timestamps.first;
 }
 
@@ -487,6 +488,22 @@ struct QueryCondition::BinaryCmpNullChecks<uint8_t*, QueryConditionOp::NE> {
   }
 };
 
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmpNullChecks<T, QueryConditionOp::ALWAYS_TRUE> {
+  static inline bool cmp(const void* lhs, uint64_t, const void*, uint64_t) {
+    return lhs != nullptr;
+  }
+};
+
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmpNullChecks<T, QueryConditionOp::ALWAYS_FALSE> {
+  static inline bool cmp(const void*, uint64_t, const void*, uint64_t) {
+    return false;
+  }
+};
+
 /** Partial template specialization for `QueryConditionOp::LT`. */
 template <typename T>
 struct QueryCondition::BinaryCmpNullChecks<T, QueryConditionOp::LT> {
@@ -651,7 +668,7 @@ void QueryCondition::apply_ast_node(
 
       if (nullable) {
         const auto& tile_validity = tile_tuple->validity_tile();
-        buffer_validity = static_cast<uint8_t*>(tile_validity.data());
+        buffer_validity = tile_validity.data_as<uint8_t>();
       }
 
       // Start the pending result cell slab at the start position
@@ -660,22 +677,16 @@ void QueryCondition::apply_ast_node(
 
       if (var_size) {
         const auto& tile = tile_tuple->var_tile();
-        const char* buffer = static_cast<char*>(tile.data());
-        const uint64_t buffer_size = tile.size();
+        const char* buffer = tile.data_as<char>();
 
         const auto& tile_offsets = tile_tuple->fixed_tile();
-        const uint64_t* buffer_offsets =
-            static_cast<uint64_t*>(tile_offsets.data());
-        const uint64_t buffer_offsets_el =
-            tile_offsets.size() / constants::cell_var_offset_size;
+        const offsets_t* buffer_offsets = tile_offsets.data_as<offsets_t>();
 
         // Iterate through each cell in this slab.
         while (c < length) {
           const uint64_t buffer_offset = buffer_offsets[start + c * stride];
           const uint64_t next_cell_offset =
-              (start + c * stride + 1 < buffer_offsets_el) ?
-                  buffer_offsets[start + c * stride + 1] :
-                  buffer_size;
+              buffer_offsets[start + c * stride + 1];
           const uint64_t cell_size = next_cell_offset - buffer_offset;
 
           const bool null_cell =
@@ -714,7 +725,7 @@ void QueryCondition::apply_ast_node(
           }
         } else {
           const auto& tile = tile_tuple->fixed_tile();
-          const char* buffer = static_cast<char*>(tile.data());
+          const char* buffer = tile.data_as<char>();
           const uint64_t cell_size = tile.cell_size();
           uint64_t buffer_offset = start * cell_size;
           const uint64_t buffer_offset_inc = stride * cell_size;
@@ -759,6 +770,30 @@ void QueryCondition::apply_ast_node(
     CombinationOp combination_op,
     std::vector<uint8_t>& result_cell_bitmap) const {
   switch (node->get_op()) {
+    case QueryConditionOp::ALWAYS_TRUE:
+      apply_ast_node<T, QueryConditionOp::ALWAYS_TRUE, CombinationOp>(
+          node,
+          fragment_metadata,
+          stride,
+          var_size,
+          nullable,
+          fill_value,
+          result_cell_slabs,
+          combination_op,
+          result_cell_bitmap);
+      break;
+    case QueryConditionOp::ALWAYS_FALSE:
+      apply_ast_node<T, QueryConditionOp::ALWAYS_FALSE, CombinationOp>(
+          node,
+          fragment_metadata,
+          stride,
+          var_size,
+          nullable,
+          fill_value,
+          result_cell_slabs,
+          combination_op,
+          result_cell_bitmap);
+      break;
     case QueryConditionOp::LT:
       apply_ast_node<T, QueryConditionOp::LT, CombinationOp>(
           node,
@@ -1097,6 +1132,8 @@ void QueryCondition::apply_ast_node(
     } break;
     case Datatype::ANY:
     case Datatype::BLOB:
+    case Datatype::GEOM_WKB:
+    case Datatype::GEOM_WKT:
     case Datatype::STRING_UTF16:
     case Datatype::STRING_UTF32:
     case Datatype::STRING_UCS2:
@@ -1104,9 +1141,8 @@ void QueryCondition::apply_ast_node(
     default:
       throw std::runtime_error(
           "QueryCondition::apply_ast_node: Cannot perform query comparison; "
-          "Unsupported query "
-          "conditional type on " +
-          node->get_field_name());
+          "Unsupported datatype " +
+          datatype_str(type) + " on " + node->get_field_name());
   }
 
   return;
@@ -1357,20 +1393,16 @@ void QueryCondition::apply_ast_node_dense(
   uint8_t* buffer_validity = nullptr;
   if (nullable) {
     const auto& tile_validity = tile_tuple->validity_tile();
-    buffer_validity = static_cast<uint8_t*>(tile_validity.data()) + src_cell;
+    buffer_validity = tile_validity.data_as<uint8_t>() + src_cell;
   }
 
   if (var_size) {
     // Get var data buffer and tile offsets buffer.
     const auto& tile = tile_tuple->var_tile();
-    const char* buffer = static_cast<char*>(tile.data());
-    const uint64_t buffer_size = tile.size();
+    const char* buffer = tile.data_as<char>();
 
     const auto& tile_offsets = tile_tuple->fixed_tile();
-    const uint64_t* buffer_offsets =
-        static_cast<uint64_t*>(tile_offsets.data());
-    const uint64_t buffer_offsets_el =
-        tile_offsets.size() / constants::cell_var_offset_size;
+    const offsets_t* buffer_offsets = tile_offsets.data_as<offsets_t>();
 
     // Iterate through each cell in this slab.
     for (uint64_t c = 0; c < result_buffer.size(); ++c) {
@@ -1378,10 +1410,8 @@ void QueryCondition::apply_ast_node_dense(
       // is string data requiring a strcmp which cannot be vectorized, this is
       // ok.
       const uint64_t offset_idx = src_cell + (start + c) * stride;
-      const uint64_t buffer_offset = buffer_offsets[offset_idx];
-      const uint64_t next_cell_offset = (offset_idx + 1 < buffer_offsets_el) ?
-                                            buffer_offsets[offset_idx + 1] :
-                                            buffer_size;
+      const offsets_t buffer_offset = buffer_offsets[offset_idx];
+      const offsets_t next_cell_offset = buffer_offsets[offset_idx + 1];
       const uint64_t cell_size = next_cell_offset - buffer_offset;
 
       // Get the cell value.
@@ -1412,7 +1442,7 @@ void QueryCondition::apply_ast_node_dense(
     } else {
       // Get the fixed size data buffers.
       const auto& tile = tile_tuple->fixed_tile();
-      const char* buffer = static_cast<char*>(tile.data());
+      const char* buffer = tile.data_as<char>();
       const uint64_t cell_size = tile.cell_size();
       uint64_t buffer_offset = ((start * stride) + src_cell) * cell_size;
       const uint64_t buffer_offset_inc = stride * cell_size;
@@ -1456,6 +1486,34 @@ void QueryCondition::apply_ast_node_dense(
     const void* cell_slab_coords,
     span<uint8_t> result_buffer) const {
   switch (node->get_op()) {
+    case QueryConditionOp::ALWAYS_TRUE:
+      apply_ast_node_dense<T, QueryConditionOp::ALWAYS_TRUE, CombinationOp>(
+          node,
+          array_schema,
+          result_tile,
+          start,
+          src_cell,
+          stride,
+          var_size,
+          nullable,
+          combination_op,
+          cell_slab_coords,
+          result_buffer);
+      break;
+    case QueryConditionOp::ALWAYS_FALSE:
+      apply_ast_node_dense<T, QueryConditionOp::ALWAYS_FALSE, CombinationOp>(
+          node,
+          array_schema,
+          result_tile,
+          start,
+          src_cell,
+          stride,
+          var_size,
+          nullable,
+          combination_op,
+          cell_slab_coords,
+          result_buffer);
+      break;
     case QueryConditionOp::LT:
       apply_ast_node_dense<T, QueryConditionOp::LT, CombinationOp>(
           node,
@@ -1606,8 +1664,7 @@ void QueryCondition::apply_ast_node_dense(
   if (nullable && node->get_value_ptr() == nullptr) {
     const auto tile_tuple = result_tile->tile_tuple(name);
     const auto& tile_validity = tile_tuple->validity_tile();
-    const auto buffer_validity =
-        static_cast<uint8_t*>(tile_validity.data()) + src_cell;
+    const auto buffer_validity = tile_validity.data_as<uint8_t>() + src_cell;
 
     // Null values can only be specified for equality operators.
     if (node->get_op() == QueryConditionOp::NE) {
@@ -1860,6 +1917,8 @@ void QueryCondition::apply_ast_node_dense(
     } break;
     case Datatype::ANY:
     case Datatype::BLOB:
+    case Datatype::GEOM_WKB:
+    case Datatype::GEOM_WKT:
     case Datatype::STRING_UTF16:
     case Datatype::STRING_UTF32:
     case Datatype::STRING_UCS2:
@@ -1867,9 +1926,8 @@ void QueryCondition::apply_ast_node_dense(
     default:
       throw std::runtime_error(
           "Cannot perform query comparison; Unsupported query conditional "
-          "type "
-          "on " +
-          node->get_field_name());
+          "type " +
+          datatype_str(type) + " on " + node->get_field_name());
   }
 }
 
@@ -2221,6 +2279,22 @@ struct QueryCondition::BinaryCmp<uint8_t*, QueryConditionOp::NE> {
   }
 };
 
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmp<T, QueryConditionOp::ALWAYS_TRUE> {
+  static inline bool cmp(const void*, uint64_t, const void*, uint64_t) {
+    return true;
+  }
+};
+
+/** Partial template specialization for `QueryConditionOp::LT`. */
+template <typename T>
+struct QueryCondition::BinaryCmp<T, QueryConditionOp::ALWAYS_FALSE> {
+  static inline bool cmp(const void*, uint64_t, const void*, uint64_t) {
+    return false;
+  }
+};
+
 /** Partial template specialization for `QueryConditionOp::LT`. */
 template <typename T>
 struct QueryCondition::BinaryCmp<T, QueryConditionOp::LT> {
@@ -2311,7 +2385,7 @@ void QueryCondition::apply_ast_node_sparse(
     ResultTile& result_tile,
     const bool var_size,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   const auto tile_tuple = result_tile.tile_tuple(node->get_field_name());
   const void* condition_value_content = node->get_value_ptr();
   const size_t condition_value_size = node->get_value_size();
@@ -2321,29 +2395,25 @@ void QueryCondition::apply_ast_node_sparse(
   if constexpr (
       std::is_same_v<CombinationOp, QCMax<BitmapType>> && nullable::value) {
     const auto& tile_validity = tile_tuple->validity_tile();
-    buffer_validity = static_cast<uint8_t*>(tile_validity.data());
+    buffer_validity = tile_validity.data_as<uint8_t>();
   }
 
   if (var_size) {
     // Get var data buffer and tile offsets buffer.
     const auto& tile = tile_tuple->var_tile();
-    const char* buffer = static_cast<char*>(tile.data());
-    const uint64_t buffer_size = tile.size();
+    const char* buffer = tile.data_as<char>();
 
     const auto& tile_offsets = tile_tuple->fixed_tile();
-    const uint64_t* buffer_offsets =
-        static_cast<uint64_t*>(tile_offsets.data());
-    const uint64_t buffer_offsets_el =
-        tile_offsets.size() / constants::cell_var_offset_size;
+    const offsets_t* buffer_offsets = tile_offsets.data_as<offsets_t>();
+    const uint64_t buffer_offsets_el = tile_offsets.size_as<offsets_t>() - 1;
 
     // Iterate through each cell.
     for (uint64_t c = 0; c < buffer_offsets_el; ++c) {
       // Check the previous cell here, which breaks vectorization but as this
       // is string data requiring a strcmp which cannot be vectorized, this is
       // ok.
-      const uint64_t buffer_offset = buffer_offsets[c];
-      const uint64_t next_cell_offset =
-          (c + 1 < buffer_offsets_el) ? buffer_offsets[c + 1] : buffer_size;
+      const offsets_t buffer_offset = buffer_offsets[c];
+      const offsets_t next_cell_offset = buffer_offsets[c + 1];
       const uint64_t cell_size = next_cell_offset - buffer_offset;
 
       // Get the cell value.
@@ -2366,7 +2436,7 @@ void QueryCondition::apply_ast_node_sparse(
   } else {
     // Get the fixed size data buffers.
     const auto& tile = tile_tuple->fixed_tile();
-    const char* buffer = static_cast<char*>(tile.data());
+    const char* buffer = tile.data_as<char>();
     const uint64_t cell_size = tile.cell_size();
     const uint64_t buffer_el = tile.size() / cell_size;
 
@@ -2403,8 +2473,24 @@ void QueryCondition::apply_ast_node_sparse(
     ResultTile& result_tile,
     const bool var_size,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   switch (node->get_op()) {
+    case QueryConditionOp::ALWAYS_TRUE:
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::ALWAYS_TRUE,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
+      break;
+    case QueryConditionOp::ALWAYS_FALSE:
+      apply_ast_node_sparse<
+          T,
+          QueryConditionOp::ALWAYS_FALSE,
+          BitmapType,
+          CombinationOp,
+          nullable>(node, result_tile, var_size, combination_op, result_bitmap);
+      break;
     case QueryConditionOp::LT:
       apply_ast_node_sparse<
           T,
@@ -2483,7 +2569,7 @@ void QueryCondition::apply_ast_node_sparse(
     const bool var_size,
     const bool nullable,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   if (nullable) {
     apply_ast_node_sparse<T, BitmapType, CombinationOp, std::true_type>(
         node, result_tile, var_size, combination_op, result_bitmap);
@@ -2499,7 +2585,7 @@ void QueryCondition::apply_ast_node_sparse(
     const ArraySchema& array_schema,
     ResultTile& result_tile,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   std::string node_field_name = node->get_field_name();
   if (!array_schema.is_field(node_field_name)) {
     std::fill(result_bitmap.begin(), result_bitmap.end(), 0);
@@ -2514,7 +2600,7 @@ void QueryCondition::apply_ast_node_sparse(
   if (nullable) {
     const auto tile_tuple = result_tile.tile_tuple(node_field_name);
     const auto& tile_validity = tile_tuple->validity_tile();
-    const auto buffer_validity = static_cast<uint8_t*>(tile_validity.data());
+    const auto buffer_validity = tile_validity.data_as<uint8_t>();
     const auto cell_num = result_tile.cell_num();
 
     if (node->get_value_ptr() == nullptr) {
@@ -2638,6 +2724,8 @@ void QueryCondition::apply_ast_node_sparse(
     } break;
     case Datatype::ANY:
     case Datatype::BLOB:
+    case Datatype::GEOM_WKB:
+    case Datatype::GEOM_WKT:
     case Datatype::STRING_UTF16:
     case Datatype::STRING_UTF32:
     case Datatype::STRING_UCS2:
@@ -2656,7 +2744,7 @@ void QueryCondition::apply_tree_sparse(
     const ArraySchema& array_schema,
     ResultTile& result_tile,
     CombinationOp combination_op,
-    std::vector<BitmapType>& result_bitmap) const {
+    tdb::pmr::vector<BitmapType>& result_bitmap) const {
   if (!node->is_expr()) {
     apply_ast_node_sparse<BitmapType>(
         node, array_schema, result_tile, combination_op, result_bitmap);
@@ -2695,7 +2783,9 @@ void QueryCondition::apply_tree_sparse(
           // Handle the cl'(q, a) case.
           // This cases on whether the combination op = OR.
         } else if constexpr (std::is_same_v<CombinationOp, QCMax<BitmapType>>) {
-          std::vector<BitmapType> combination_op_bitmap(result_bitmap_size, 1);
+          auto resource = result_bitmap.get_allocator().resource();
+          tdb::pmr::vector<BitmapType> combination_op_bitmap(
+              result_bitmap_size, 1, resource);
 
           for (const auto& child : node->get_children()) {
             apply_tree_sparse<BitmapType>(
@@ -2717,7 +2807,9 @@ void QueryCondition::apply_tree_sparse(
          *                        = a /\ (cl1'(q; cl2'(q; 0)))
          */
       case QueryConditionCombinationOp::OR: {
-        std::vector<BitmapType> combination_op_bitmap(result_bitmap_size, 0);
+        auto resource = result_bitmap.get_allocator().resource();
+        tdb::pmr::vector<BitmapType> combination_op_bitmap(
+            result_bitmap_size, 0, resource);
 
         for (const auto& child : node->get_children()) {
           apply_tree_sparse<BitmapType>(
@@ -2748,7 +2840,7 @@ template <typename BitmapType>
 Status QueryCondition::apply_sparse(
     const ArraySchema& array_schema,
     ResultTile& result_tile,
-    std::vector<BitmapType>& result_bitmap) {
+    tdb::pmr::vector<BitmapType>& result_bitmap) {
   apply_tree_sparse<BitmapType>(
       tree_,
       array_schema,
@@ -2775,14 +2867,9 @@ uint64_t QueryCondition::condition_index() const {
   return condition_index_;
 }
 
-void QueryCondition::set_ast(tdb_unique_ptr<ASTNode>&& ast) {
-  tree_ = std::move(ast);
-}
-
 // Explicit template instantiations.
 template Status QueryCondition::apply_sparse<uint8_t>(
-    const ArraySchema& array_schema, ResultTile&, std::vector<uint8_t>&);
+    const ArraySchema& array_schema, ResultTile&, tdb::pmr::vector<uint8_t>&);
 template Status QueryCondition::apply_sparse<uint64_t>(
-    const ArraySchema& array_schema, ResultTile&, std::vector<uint64_t>&);
-}  // namespace sm
-}  // namespace tiledb
+    const ArraySchema& array_schema, ResultTile&, tdb::pmr::vector<uint64_t>&);
+}  // namespace tiledb::sm
