@@ -37,6 +37,7 @@
 #include "tiledb/sm/array/array.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/array_schema/dimension.h"
+#include "tiledb/sm/consolidator/consolidator.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/comparators.h"
 #include "tiledb/sm/misc/hilbert.h"
@@ -73,43 +74,31 @@ class GlobalOrderWriterStatusException : public StatusException {
 GlobalOrderWriter::GlobalOrderWriter(
     stats::Stats* stats,
     shared_ptr<Logger> logger,
-    StorageManager* storage_manager,
-    Array* array,
-    Config& config,
-    std::unordered_map<std::string, QueryBuffer>& buffers,
-    Subarray& subarray,
-    Layout layout,
+    StrategyParams& params,
     uint64_t fragment_size,
     std::vector<WrittenFragmentInfo>& written_fragment_info,
     bool disable_checks_consolidation,
     std::vector<std::string>& processed_conditions,
     Query::CoordsInfo& coords_info,
     bool remote_query,
-    optional<std::string> fragment_name,
-    bool skip_checks_serialization)
+    optional<std::string> fragment_name)
     : WriterBase(
           stats,
           logger,
-          storage_manager,
-          array,
-          config,
-          buffers,
-          subarray,
-          layout,
+          params,
           written_fragment_info,
           disable_checks_consolidation,
           coords_info,
           remote_query,
-          fragment_name,
-          skip_checks_serialization)
+          fragment_name)
     , processed_conditions_(processed_conditions)
     , fragment_size_(fragment_size)
     , current_fragment_size_(0) {
   // Check the layout is global order.
-  if (layout != Layout::GLOBAL_ORDER) {
+  if (layout_ != Layout::GLOBAL_ORDER) {
     throw GlobalOrderWriterStatusException(
         "Failed to initialize global order writer. Layout " +
-        layout_str(layout) + " is not global order.");
+        layout_str(layout_) + " is not global order.");
   }
 
   // Check no ordered attributes.
@@ -188,7 +177,7 @@ Status GlobalOrderWriter::alloc_global_write_state() {
   global_write_state_.reset(new GlobalWriteState);
 
   // Alloc FragmentMetadata object
-  global_write_state_->frag_meta_ = make_shared<FragmentMetadata>(HERE());
+  global_write_state_->frag_meta_ = this->create_fragment_metadata();
   // Used in serialization when FragmentMetadata is built from ground up
   global_write_state_->frag_meta_->set_context_resources(
       &storage_manager_->resources());
@@ -209,20 +198,18 @@ Status GlobalOrderWriter::init_global_write_state() {
     const auto capacity = array_schema_.capacity();
     const auto cell_num_per_tile =
         coords_info_.has_coords_ ? capacity : domain.cell_num_per_tile();
-    auto last_tile_vector = std::pair<std::string, WriterTileTupleVector>(
-        name, WriterTileTupleVector());
-    try {
-      last_tile_vector.second.emplace_back(WriterTileTuple(
-          array_schema_,
-          cell_num_per_tile,
-          var_size,
-          nullable,
-          cell_size,
-          type));
-    } catch (const std::logic_error& le) {
-      return Status_WriterError(le.what());
-    }
-    global_write_state_->last_tiles_.emplace(std::move(last_tile_vector));
+    auto last_tiles_it = global_write_state_->last_tiles_.emplace(
+        std::piecewise_construct,
+        std::forward_as_tuple(name),
+        std::forward_as_tuple());
+    last_tiles_it.first->second.emplace_back(
+        array_schema_,
+        cell_num_per_tile,
+        var_size,
+        nullable,
+        cell_size,
+        type,
+        query_memory_tracker_);
 
     // Initialize cells written
     global_write_state_->cells_written_[name] = 0;
@@ -725,8 +712,11 @@ Status GlobalOrderWriter::finalize_global_write_state() {
     commit_uris.emplace_back(commit_uri);
 
     auto write_version = array_->array_schema_latest().write_version();
-    storage_manager_->write_consolidated_commits_file(
-        write_version, array_->array_directory(), commit_uris);
+    Consolidator::write_consolidated_commits_file(
+        write_version,
+        array_->array_directory(),
+        commit_uris,
+        storage_manager_);
   }
 
   // Delete global write state
@@ -874,7 +864,10 @@ Status GlobalOrderWriter::prepare_full_tiles(
 
   // Initialize attribute and coordinate tiles
   for (const auto& it : buffers_) {
-    (*tiles)[it.first] = WriterTileTupleVector();
+    (*tiles).emplace(
+        std::piecewise_construct,
+        std::forward_as_tuple(it.first),
+        std::forward_as_tuple());
   }
 
   auto num = buffers_.size();
@@ -976,16 +969,26 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
   if (full_tile_num > 0) {
     tiles->reserve(full_tile_num);
     for (uint64_t i = 0; i < full_tile_num; i++) {
-      tiles->emplace_back(WriterTileTuple(
-          array_schema_, cell_num_per_tile, false, nullable, cell_size, type));
+      tiles->emplace_back(
+          array_schema_,
+          cell_num_per_tile,
+          false,
+          nullable,
+          cell_size,
+          type,
+          query_memory_tracker_);
     }
 
     // Handle last tile (it must be either full or empty)
     auto tile_it = tiles->begin();
     if (last_tile_cell_idx == cell_num_per_tile) {
-      tile_it->fixed_tile().swap(last_tile.fixed_tile());
+      tile_it->fixed_tile().write(
+          last_tile.fixed_tile().data(), 0, last_tile.fixed_tile().size());
       if (nullable) {
-        tile_it->validity_tile().swap(last_tile.validity_tile());
+        tile_it->validity_tile().write(
+            last_tile.validity_tile().data(),
+            0,
+            last_tile.validity_tile().size());
       }
       tile_it++;
     } else if (last_tile_cell_idx != 0) {
@@ -1169,8 +1172,6 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
         ++cell_idx;
       } while (last_tile_cell_idx != cell_num_per_tile && cell_idx != cell_num);
     }
-
-    last_tile.var_tile().set_size(last_var_offset);
   }
 
   // Initialize full tiles and set previous last tile as first tile
@@ -1183,19 +1184,33 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
   if (full_tile_num > 0) {
     tiles->reserve(full_tile_num);
     for (uint64_t i = 0; i < full_tile_num; i++) {
-      tiles->emplace_back(WriterTileTuple(
-          array_schema_, cell_num_per_tile, true, nullable, cell_size, type));
+      tiles->emplace_back(
+          array_schema_,
+          cell_num_per_tile,
+          true,
+          nullable,
+          cell_size,
+          type,
+          query_memory_tracker_);
     }
 
     // Handle last tile (it must be either full or empty)
     auto tile_it = tiles->begin();
     if (last_tile_cell_idx == cell_num_per_tile) {
-      last_var_offset = 0;
-      tile_it->offset_tile().swap(last_tile.offset_tile());
-      tile_it->var_tile().swap(last_tile.var_tile());
+      tile_it->offset_tile().write(
+          last_tile.offset_tile().data(), 0, last_tile.offset_tile().size());
+      tile_it->var_tile().write_var(
+          last_tile.var_tile().data(), 0, last_var_offset);
+      tile_it->var_tile().set_size(last_var_offset);
       if (nullable) {
-        tile_it->validity_tile().swap(last_tile.validity_tile());
+        tile_it->validity_tile().write(
+            last_tile.validity_tile().data(),
+            0,
+            last_tile.validity_tile().size());
       }
+
+      last_var_offset = 0;
+
       tile_it++;
     } else if (last_tile_cell_idx != 0) {
       return Status_WriterError(
