@@ -73,42 +73,30 @@ class UnorderWriterException : public StatusException {
 UnorderedWriter::UnorderedWriter(
     stats::Stats* stats,
     shared_ptr<Logger> logger,
-    StorageManager* storage_manager,
-    Array* array,
-    Config& config,
-    std::unordered_map<std::string, QueryBuffer>& buffers,
-    Subarray& subarray,
-    Layout layout,
+    StrategyParams& params,
     std::vector<WrittenFragmentInfo>& written_fragment_info,
     Query::CoordsInfo& coords_info,
     std::unordered_set<std::string>& written_buffers,
     bool remote_query,
-    optional<std::string> fragment_name,
-    bool skip_checks_serialization)
+    optional<std::string> fragment_name)
     : WriterBase(
           stats,
           logger,
-          storage_manager,
-          array,
-          config,
-          buffers,
-          subarray,
-          layout,
+          params,
           written_fragment_info,
           false,
           coords_info,
           remote_query,
-          fragment_name,
-          skip_checks_serialization)
+          fragment_name)
     , frag_uri_(std::nullopt)
     , written_buffers_(written_buffers)
     , is_coords_pass_(true) {
   // Check the layout is unordered.
-  if (layout != Layout::UNORDERED) {
+  if (layout_ != Layout::UNORDERED) {
     throw UnorderWriterException(
         "Failed to initialize UnorderedWriter; The unordered writer does not "
         "support layout " +
-        layout_str(layout));
+        layout_str(layout_));
   }
 
   // Check the array is sparse.
@@ -179,7 +167,7 @@ std::string UnorderedWriter::name() {
 
 Status UnorderedWriter::alloc_frag_meta() {
   // Alloc FragmentMetadata object.
-  frag_meta_ = make_shared<FragmentMetadata>(HERE());
+  frag_meta_ = this->create_fragment_metadata();
   // Used in serialization when FragmentMetadata is built from ground up.
   frag_meta_->set_context_resources(&storage_manager_->resources());
 
@@ -392,7 +380,10 @@ Status UnorderedWriter::prepare_tiles(
   for (const auto& it : buffers_) {
     const auto& name = it.first;
     if (written_buffers_.count(name) == 0) {
-      (*tiles).emplace(name, WriterTileTupleVector());
+      (*tiles).emplace(
+          std::piecewise_construct,
+          std::forward_as_tuple(name),
+          std::forward_as_tuple());
     }
   }
 
@@ -401,8 +392,8 @@ Status UnorderedWriter::prepare_tiles(
       storage_manager_->compute_tp(), 0, tiles->size(), [&](uint64_t i) {
         auto tiles_it = tiles->begin();
         std::advance(tiles_it, i);
-        const auto& name = tiles_it->first;
-        RETURN_CANCEL_OR_ERROR(prepare_tiles(name, &((*tiles)[name])));
+        RETURN_CANCEL_OR_ERROR(
+            prepare_tiles(tiles_it->first, &(tiles_it->second)));
         return Status::Ok();
       });
 
@@ -439,8 +430,14 @@ Status UnorderedWriter::prepare_tiles_fixed(
   // Initialize tiles
   tiles->reserve(tile_num);
   for (uint64_t i = 0; i < tile_num; i++) {
-    tiles->emplace_back(WriterTileTuple(
-        array_schema_, cell_num_per_tile, false, nullable, cell_size, type));
+    tiles->emplace_back(
+        array_schema_,
+        cell_num_per_tile,
+        false,
+        nullable,
+        cell_size,
+        type,
+        query_memory_tracker_);
   }
 
   // Write all cells one by one
@@ -510,8 +507,14 @@ Status UnorderedWriter::prepare_tiles_var(
   // Initialize tiles
   tiles->reserve(tile_num);
   for (uint64_t i = 0; i < tile_num; i++) {
-    tiles->emplace_back(WriterTileTuple(
-        array_schema_, cell_num_per_tile, true, nullable, cell_size, type));
+    tiles->emplace_back(
+        array_schema_,
+        cell_num_per_tile,
+        true,
+        nullable,
+        cell_size,
+        type,
+        query_memory_tracker_);
   }
 
   // Write all cells one by one
@@ -665,7 +668,7 @@ Status UnorderedWriter::unordered_write() {
     }
 
     // Create new fragment
-    frag_meta_ = make_shared<FragmentMetadata>(HERE());
+    frag_meta_ = this->create_fragment_metadata();
     RETURN_CANCEL_OR_ERROR(create_fragment(false, frag_meta_));
   }
 
