@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,6 +40,7 @@
 #include "tiledb/sm/filesystem/vfs.h"
 #include "tiledb/sm/filter/compression_filter.h"
 #include "tiledb/sm/filter/webp_filter.h"
+#include "tiledb/sm/fragment/fragment_identifier.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/comparators.h"
 #include "tiledb/sm/misc/hilbert.h"
@@ -75,27 +76,13 @@ class WriterBaseStatusException : public StatusException {
 WriterBase::WriterBase(
     stats::Stats* stats,
     shared_ptr<Logger> logger,
-    StorageManager* storage_manager,
-    Array* array,
-    Config& config,
-    std::unordered_map<std::string, QueryBuffer>& buffers,
-    Subarray& subarray,
-    Layout layout,
+    StrategyParams& params,
     std::vector<WrittenFragmentInfo>& written_fragment_info,
     bool disable_checks_consolidation,
     Query::CoordsInfo& coords_info,
     bool remote_query,
-    optional<std::string> fragment_name,
-    bool skip_checks_serialization)
-    : StrategyBase(
-          stats,
-          logger->clone("Writer", ++logger_id_),
-          storage_manager,
-          array,
-          config,
-          buffers,
-          subarray,
-          layout)
+    optional<std::string> fragment_name)
+    : StrategyBase(stats, logger->clone("Writer", ++logger_id_), params)
     , disable_checks_consolidation_(disable_checks_consolidation)
     , coords_info_(coords_info)
     , check_coord_dups_(false)
@@ -110,7 +97,7 @@ WriterBase::WriterBase(
         "Cannot initialize query; Storage manager not set");
   }
 
-  if (!skip_checks_serialization && buffers_.empty()) {
+  if (!params.skip_checks_serialization() && buffers_.empty()) {
     throw WriterBaseStatusException(
         "Cannot initialize writer; Buffers not set");
   }
@@ -202,7 +189,7 @@ WriterBase::WriterBase(
     check_extra_element();
   }
 
-  if (!skip_checks_serialization) {
+  if (!params.skip_checks_serialization()) {
     // Consolidation might set a subarray that is not tile aligned.
     if (!disable_checks_consolidation) {
       check_subarray();
@@ -227,8 +214,8 @@ WriterBase::WriterBase(
   auto frag_dir_uri =
       array_->array_directory().get_fragments_dir(write_version);
   fragment_uri_ = frag_dir_uri.join_path(new_fragment_str);
-  throw_if_not_ok(utils::parse::get_timestamp_range(
-      fragment_uri_, &fragment_timestamp_range_));
+  FragmentID fragment_id{fragment_uri_};
+  fragment_timestamp_range_ = fragment_id.timestamp_range();
 }
 
 WriterBase::~WriterBase() {
@@ -322,6 +309,11 @@ void WriterBase::refresh_config() {
 /*          PRIVATE METHODS       */
 /* ****************************** */
 
+shared_ptr<FragmentMetadata> WriterBase::create_fragment_metadata() {
+  return make_shared<FragmentMetadata>(
+      HERE(), &storage_manager_->resources(), array_memory_tracker_);
+}
+
 Status WriterBase::add_written_fragment_info(const URI& uri) {
   written_fragment_info_.emplace_back(uri, fragment_timestamp_range_);
   return Status::Ok();
@@ -718,7 +710,7 @@ Status WriterBase::compute_tiles_metadata(
       auto tiles_it = tiles.begin();
       std::advance(tiles_it, i);
       const auto& attr = tiles_it->first;
-      auto& attr_tiles = tiles[attr];
+      auto& attr_tiles = tiles.at(attr);
       const auto type = array_schema_.type(attr);
       const auto is_dim = array_schema_.is_dim(attr);
       const auto var_size = array_schema_.var_size(attr);
@@ -800,10 +792,10 @@ Status WriterBase::create_fragment(
   frag_meta = make_shared<FragmentMetadata>(
       HERE(),
       &storage_manager_->resources(),
-      nullptr,
       array_->array_schema_latest_ptr(),
       fragment_uri_,
       timestamp_range,
+      array_memory_tracker_,
       dense,
       has_timestamps,
       has_delete_metadata);
@@ -959,8 +951,14 @@ Status WriterBase::init_tiles(
       coords_info_.has_coords_ ? capacity : domain.cell_num_per_tile();
   tiles->reserve(tile_num);
   for (uint64_t i = 0; i < tile_num; i++) {
-    tiles->emplace_back(WriterTileTuple(
-        array_schema_, cell_num_per_tile, var_size, nullable, cell_size, type));
+    tiles->emplace_back(
+        array_schema_,
+        cell_num_per_tile,
+        var_size,
+        nullable,
+        cell_size,
+        type,
+        query_memory_tracker_);
   }
 
   return Status::Ok();
