@@ -64,9 +64,7 @@ namespace serialization {
 Status fragment_metadata_from_capnp(
     const shared_ptr<const ArraySchema>& array_schema,
     const capnp::FragmentMetadata::Reader& frag_meta_reader,
-    shared_ptr<FragmentMetadata> frag_meta,
-    ContextResources* resources = nullptr,
-    MemoryTracker* memory_tracker = nullptr);
+    shared_ptr<FragmentMetadata> frag_meta);
 
 /**
  * Serialize Fragment Metadata sizes and offsets
@@ -103,4 +101,4 @@ Status fragment_metadata_to_capnp(
 }  // namespace sm
 }  // namespace tiledb
 
-#endif  // TILEDB_SERIALIZATION_FRAGMENT_METADATA_H
\ No newline at end of file
+#endif  // TILEDB_SERIALIZATION_FRAGMENT_METADATA_H
