@@ -333,7 +333,14 @@ class SubarrayPartitioner {
   Subarray& subarray();
 
   /** Returns `stats_`. */
-  stats::Stats* stats() const;
+  const stats::Stats& stats() const;
+
+  /** Populate the owned stats instance with data.
+   * To be removed when the class will get a C41 constructor.
+   *
+   * @param data Data to populate the stats with.
+   */
+  void set_stats(const stats::StatsData& data);
 
  private:
   /* ********************************* */
